@@ -18,19 +18,28 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: Communications.cc,v 3.14 2007/02/21 10:21:15 rousse Exp $
+// $Id: Communications.cc,v 3.18 2007/11/05 14:30:03 rousse Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
 #include "Communications.hh"
-
+#include <assert.h>
 #include "PrettyDebug.hh"
 
-#include <fstream>
-#include <iostream>
-#include <assert.h>
-#include <unistd.h>
-#include <errno.h>
+#ifdef _WIN32
+    #include <windows.h>
+	#ifdef max
+		#undef max
+	#endif
+#else
+    #include <fstream>
+    #include <iostream>
+    #include <unistd.h>
+	#include <errno.h>
+#endif
+
+
+
 
 using std::ifstream ;
 using std::ios ;
@@ -42,6 +51,7 @@ namespace certi {
 namespace rtia {
 
 static pdCDebug D("RTIA_COMM", "(RTIA Comm) ");
+static PrettyDebug G("GENDOC",__FILE__);
 
 // ----------------------------------------------------------------------------
 /*! Wait a message coming from RTIG. Parameters are :
@@ -142,12 +152,15 @@ Communications::~Communications()
 void
 Communications::requestFederateService(Message *req, Message *rep)
 {
+    // G.Out(pdGendoc,"enter Communications::requestFederateService for message "
+    //               "type %d",req->type);
     assert(req != NULL);
     D.Out(pdRequest, "Sending Request to Federate, Type %d.", req->type);
     sendUN(req);
     receiveUN(rep);
     D.Out(pdAnswer, "Received Answer from Federate.");
     assert(req->type == rep->type);
+    // G.Out(pdGendoc,"exit  Communications::requestFederateService");
 }
 
 // ----------------------------------------------------------------------------
@@ -182,14 +195,23 @@ Communications::readMessage(int &n, NetworkMessage *msg_reseau, Message *msg)
     FD_SET(tcp_fd, &fdset);
     FD_SET(udp_fd, &fdset);
 
-    int max_fd = std::max(_socket_un, std::max(tcp_fd, udp_fd));
+	#ifdef _WIN32	//For Windows, First "Select" argument isn't used
+		int	max_fd= 0;
+	#else
+		int max_fd = std::max(_socket_un, std::max(tcp_fd, udp_fd));
+	#endif
 
 #ifdef FEDERATION_USES_MULTICAST
     // if multicast link is initialized (during join federation).
     if (_est_init_mc) {
         FD_SET(_socket_mc, &fdset);
-        max_fd = std::max(max_fd, _socket_mc);
-    }
+
+ 		#ifdef _WIN32	//For Windows, First "Select" argument isn't used
+			max_fd= 0;
+		#else
+			max_fd = std::max(max_fd, _socket_mc);
+		#endif
+   }
 #endif
 
     if (!waitingList.empty()) {
@@ -223,9 +245,13 @@ Communications::readMessage(int &n, NetworkMessage *msg_reseau, Message *msg)
         // waitingList is empty and no data in TCP buffer.
         // Wait a message (coming from federate or network).
         if (select(max_fd, &fdset, NULL, NULL, NULL) < 0) {
-            if (errno == EINTR)
+			#ifdef _WIN32							//dotNet
+				 if(WSAGetLastError() == WSAEINTR)
+			#else
+				 if(errno == EINTR)
+			#endif
                 throw NetworkSignal("");
-            else
+				else
                 throw NetworkError("");
         }
 
@@ -316,4 +342,4 @@ Communications::receiveUN(Message *Msg)
 
 }} // namespace certi/rtia
 
-// $Id: Communications.cc,v 3.14 2007/02/21 10:21:15 rousse Exp $
+// $Id: Communications.cc,v 3.18 2007/11/05 14:30:03 rousse Exp $
