@@ -18,7 +18,7 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: FederationManagement.cc,v 3.22 2007/04/24 15:10:31 erk Exp $
+// $Id: FederationManagement.cc,v 3.36 2007/11/19 10:20:53 erk Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -27,12 +27,16 @@
 #include "TimeManagement.hh"
 #include "PrettyDebug.hh"
 
+#ifdef _WIN32
+#include <windows.h>
+#else
 #include <list>
 #include <cstdio>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdlib.h>
+#endif
 #include <assert.h>
 
 using std::list ;
@@ -42,6 +46,7 @@ namespace certi {
 namespace rtia {
 
 static pdCDebug D("RTIA_FM", "(RTIA FM) ");
+static PrettyDebug G("GENDOC",__FILE__);
 
 // ----------------------------------------------------------------------------
 //! FederationManagement.
@@ -85,7 +90,7 @@ FederationManagement::~FederationManagement()
 
         destroyFederationExecution(_nom_federation, e);
         while (e != e_NO_EXCEPTION) {
-            sleep(1);
+				sleep(1);
             destroyFederationExecution(_nom_federation, e);
         }
         cout << "RTIA: Federation destroyed" << endl ;
@@ -99,11 +104,12 @@ FederationManagement::
 createFederationExecution(const char *theName,
                           TypeException &e)
          throw ( FederationExecutionAlreadyExists,
-                 CouldNotOpenFED,
+                 CouldNotOpenFED,ErrorReadingFED,
                  RTIinternalError)
 {
     NetworkMessage requete, reponse ;
 
+    G.Out(pdGendoc,"enter FederationManagement::createFederationExecution");
     D.Out(pdInit, "Creating Federation %s.", theName);
 
     e = e_NO_EXCEPTION ;
@@ -112,32 +118,60 @@ createFederationExecution(const char *theName,
     if (_est_createur_federation || _est_membre_federation)
         e = e_RTIinternalError ;
 
-    if (e == e_NO_EXCEPTION) {
+    if (e == e_NO_EXCEPTION)
+        {
         requete.type = NetworkMessage::CREATE_FEDERATION_EXECUTION ;
         strcpy(requete.federationName, theName);
         strcpy(requete.FEDid, _FEDid) ;
 
+        G.Out(pdGendoc,"createFederationExecution====>send Message to RTIG");
+
         comm->sendMessage(&requete);
 
         comm->waitMessage(&reponse, NetworkMessage::CREATE_FEDERATION_EXECUTION,
                           federate);
-
-        if (reponse.exception == e_NO_EXCEPTION) {
+        // We have to see if C_F_E is OK.
+        if (reponse.exception == e_NO_EXCEPTION)
+            {
             strcpy(_nom_federation, theName);
             _numero_federation = reponse.federation ;
             _est_createur_federation = true ;
             D.Out(pdInit, "est createur");
-        }
+            }
         else if (reponse.exception == e_CouldNotOpenFED)
+            // RTIG encounters a problem creating federation execution
             {
             e = reponse.exception;
-            cout << "Could not open FED." << endl;
+            G.Out(pdGendoc,"exit FederationManagement::"
+                           "createFederationExecution on exception");
+            throw CouldNotOpenFED (reponse.exceptionReason) ;
             }           
-        else {
+        else if (reponse.exception == e_FederationExecutionAlreadyExists)
+            {
+            e = reponse.exception;
+            G.Out(pdGendoc,"exit FederationManagement::"
+                           "createFederationExecution on exception");
+            throw FederationExecutionAlreadyExists (reponse.exceptionReason) ;
+            }
+        else if (reponse.exception == e_ErrorReadingFED)
+            {
+            e = reponse.exception;
+            G.Out(pdGendoc,"exit FederationManagement::"
+                           "createFederationExecution on exception ErrorReadingFED");
+            throw ErrorReadingFED (reponse.exceptionReason) ;
+            }
+        else
+            {
             e = reponse.exception ;
+            G.Out(pdGendoc,"exit FederationManagement::"
+                           "createFederationExecution on exception RTIinternalError");
+            throw RTIinternalError (reponse.exceptionReason) ;
             D.Out(pdInit, "deja cree");
+            }
         }
-    }
+
+    G.Out(pdGendoc,"exit FederationManagement::createFederationExecution");
+
 }
 
 
@@ -175,6 +209,12 @@ destroyFederationExecution(const char *theName,
             _numero_federation = 0 ;
             _est_createur_federation = false ;
             _fin_execution = true ;
+            // Now, remove temporary file (if not yet done)
+            if ( _FEDid[0] != '\0' )
+                {
+                std::remove(_FEDid);
+                _FEDid[0] = '\0' ;
+                }
         }
         else
             e = reponse.exception ;
@@ -191,8 +231,9 @@ joinFederationExecution(const char *Federate,
 {
     NetworkMessage requete, reponse, requeteFED ;
     int i, nb ;
-    char filename[MAX_FEDFILE_NAME_LENGTH] ; // Pour s'en servir pour le nom du fichier temporaire
+    char filename[MAX_FEDFILE_NAME_LENGTH] ; // Needed for working file name
 
+    G.Out(pdGendoc,"enter FederationManagement::joinFederationExecution");
     D.Out(pdInit, "Join Federation %s as %s.", Federation, Federate);
 
     e = e_NO_EXCEPTION ;
@@ -211,6 +252,8 @@ joinFederationExecution(const char *Federate,
         requete.bestEffortAddress = comm->getAddress();
         requete.bestEffortPeer = comm->getPort();
 
+        G.Out(pdGendoc,"joinFederationExecution====>send Message to RTIG");
+
         comm->sendMessage(&requete);
 
         // Waiting RTIG answer for FED file opened
@@ -225,13 +268,14 @@ joinFederationExecution(const char *Federate,
             {
 	    stat->rtiService(NetworkMessage::GET_FED_FILE);
             // RTIA have to open a new file for working
-            // We have to build a name for working file, name begins by :RTIA
-            strcpy(filename,":RTIA:");
+            // We have to build a name for working file, name begins by _RTIA_
+            strcpy(filename,"_RTIA_");
+
             // First pid converted in char and added
             // Then federation name added also
             // Last file type : fed or xml ?
             char pid_name[10];
-            sprintf(pid_name,"%d:",getpid());
+            sprintf(pid_name,"%d_",getpid());
             strcat(filename,pid_name); 
             strcat(filename,Federation);     
             string filename_RTIG = reponse.FEDid ;
@@ -261,7 +305,10 @@ joinFederationExecution(const char *Federate,
             if ( e == e_NO_EXCEPTION)
                 requeteFED.number = 0 ;  // OK for open
             else
-                requeteFED.number = 1 ;  
+                requeteFED.number = 1 ; 
+
+            G.Out(pdGendoc,"joinFederationExecution====> begin FED file get from RTIG");
+ 
             comm->sendMessage(&requeteFED);
     
             // Now read loop from RTIG to get line contents and then write it into file
@@ -298,7 +345,9 @@ joinFederationExecution(const char *Federate,
             // close working file
             fclose(fdd); 
             }                          
-            
+ 
+        G.Out(pdGendoc,"joinFederationExecution====> end FED file get from RTIG"); 
+          
          // Waiting RTIG answer (from any federate)
         comm->waitMessage(&reponse, NetworkMessage::JOIN_FEDERATION_EXECUTION, 0);
 
@@ -325,12 +374,13 @@ joinFederationExecution(const char *Federate,
             }
 
             _est_membre_federation = true ;
+            G.Out(pdGendoc,"exit(%d) FederationManagement::joinFederationExecution",federate);
             return(federate);
         }
         else
             e = reponse.exception ;
     }
-
+    G.Out(pdGendoc,"exit(0) FederationManagement::joinFederationExecution");
     return(0);
 }
 
@@ -367,6 +417,13 @@ FederationManagement::resignFederationExecution(RTI::ResignAction,
         _est_membre_federation = false ;
         _numero_federation = 0 ;
         federate = 0 ;
+        // Now, remove temporary file (if not yet done)
+            if ( _FEDid[0] != '\0' )
+                {
+                std::remove(_FEDid);
+                _FEDid[0] = '\0' ;
+                }
+
 
         // BUG: Voir DestroyFederation ou ~GF.
         if (!_est_createur_federation)
@@ -382,6 +439,7 @@ FederationManagement::registerSynchronization(const char *label,
                                               TypeException &e)
 {
     D.Out(pdProtocol, "RegisterSynchronization.");
+    G.Out(pdGendoc,"enter FederationManagement::registerSynchronization");
 
     assert(label != NULL);
 
@@ -410,10 +468,71 @@ FederationManagement::registerSynchronization(const char *label,
         req.federate = federate ;
         req.setLabel(label);
         req.setTag(tag);
+        // no federates set so boolean must be false
+        req.setBoolean(false) ;
+
+        G.Out(pdGendoc,"      registerSynchronization====> send Message to RTIG");
+
         comm->sendMessage(&req);
     }
+
+    G.Out(pdGendoc,"exit  FederationManagement::registerSynchronization");
+
 }
 
+// ----------------------------------------------------------------------------
+//! Register synchronization with set of federates
+void
+FederationManagement::registerSynchronization(const char *label,
+                                              const char *tag,
+                                              unsigned short array_size,
+                                              FederateHandle *fed_array,
+                                              TypeException &e)
+{
+    D.Out(pdProtocol, "RegisterSynchronization.");
+    G.Out(pdGendoc,"enter FederationManagement::registerSynchronization with federate set");
+
+    assert(label != NULL);
+
+    e = e_NO_EXCEPTION ;
+
+    list<char *>::const_iterator i = synchronizationLabels.begin();
+    bool exists = false ;
+    for (; i != synchronizationLabels.end(); i++) {
+        if (!strcmp((*i), label)) {
+            e = e_FederationAlreadyPaused ; // Label already pending.
+            exists = true ;
+            break ;
+        }
+    }
+
+    if (!exists)
+        synchronizationLabels.push_back(strdup(label));
+
+    if (!_est_membre_federation)
+        e = e_FederateNotExecutionMember ;
+
+    if (e == e_NO_EXCEPTION) {
+        NetworkMessage req ;
+        req.type = NetworkMessage::REGISTER_FEDERATION_SYNCHRONIZATION_POINT ;
+        req.federation = _numero_federation ;
+        req.federate = federate ;
+        req.setLabel(label);
+        req.setTag(tag);
+        // federates set exists so boolean must be true
+        req.setBoolean(true) ;
+        req.handleArraySize = array_size ;
+        for ( int j=0 ; j < array_size ; j++)
+            req.handleArray[j] = fed_array[j] ;
+
+        G.Out(pdGendoc,"      registerSynchronization====> send Message to RTIG");
+
+        comm->sendMessage(&req);
+    }
+
+    G.Out(pdGendoc,"exit  FederationManagement::registerSynchronization with federate set");
+
+}
 // ----------------------------------------------------------------------------
 //! Unregister synchronization.
 void
@@ -493,6 +612,28 @@ FederationManagement::announceSynchronizationPoint(const char *label,
     comm->requestFederateService(&req, &rep);
 }
 
+// ----------------------------------------------------------------------------
+void FederationManagement::
+synchronizationPointRegistrationFailed(const char *label)
+{
+    D.Out(pdInit, "Synchronization Point Registration Failed \"%s\".",
+          label);
+
+    Message req, rep ;
+
+    G.Out(pdGendoc,"enter FederationManagement::synchronizationPointRegistrationFailed");
+
+    assert(label != NULL);
+
+    req.type = Message::SYNCHRONIZATION_POINT_REGISTRATION_FAILED ;
+    req.setLabel(label);
+
+    comm->requestFederateService(&req, &rep);
+
+    G.Out(pdGendoc,"exit  FederationManagement::synchronizationPointRegistrationFailed");
+
+}
+
 // ----------------------------------------------------------------------------
 void FederationManagement::
 synchronizationPointRegistrationSucceeded(const char *label)
@@ -502,12 +643,17 @@ synchronizationPointRegistrationSucceeded(const char *label)
 
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter FederationManagement::synchronizationPointRegistrationSucceeded");
+
     assert(label != NULL);
 
     req.type = Message::SYNCHRONIZATION_POINT_REGISTRATION_SUCCEEDED ;
     req.setLabel(label);
 
     comm->requestFederateService(&req, &rep);
+
+    G.Out(pdGendoc,"exit  FederationManagement::synchronizationPointRegistrationSucceeded");
+
 }
 
 // ----------------------------------------------------------------------------
@@ -527,12 +673,15 @@ FederationManagement::federationSynchronized(const char *label)
 }
 
 // ----------------------------------------------------------------------------
+// requestFederationSave with time
 void
 FederationManagement::requestFederationSave(const char *label,
                                             FederationTime the_time,
-                                            TypeException &)
+                                            TypeException &e)
 {
     D.Out(pdInit, "Request for federation save \"%s\".", label);
+    G.Out(pdGendoc,"enter FederationManagement::requestFederationSave "
+                   "with time");
 
     assert(label != 0);
 
@@ -540,6 +689,13 @@ FederationManagement::requestFederationSave(const char *label,
     req.type = NetworkMessage::REQUEST_FEDERATION_SAVE ;
     req.date = the_time ;
     req.setLabel(label);
+    req.federation = _numero_federation ;
+    req.federate = federate ;
+    // boolean true because with time
+    req.setBoolean(true);
+
+    G.Out(pdGendoc,"      requestFederationSave====>send Message R_F_S to RTIG");
+
     comm->sendMessage(&req);
 
     // Should make sure that RTIG don't have any save or restore recently set.
@@ -548,6 +704,34 @@ FederationManagement::requestFederationSave(const char *label,
     // e = rep.exception ;
 }
 
+// ----------------------------------------------------------------------------
+// requestFederationSave without time
+void
+FederationManagement::requestFederationSave(const char *label,
+                                            TypeException &e)
+{
+    D.Out(pdInit, "Request for federation save \"%s\".", label);
+    G.Out(pdGendoc,"enter FederationManagement::requestFederationSave "
+                   "without time");
+
+    assert(label != 0);
+
+    NetworkMessage req ;
+    req.type = NetworkMessage::REQUEST_FEDERATION_SAVE ;
+    req.setLabel(label);
+    req.federation = _numero_federation ;
+    req.federate = federate ;
+    // boolean false because without time
+    req.setBoolean(false);
+
+    G.Out(pdGendoc,"      requestFederationSave====>send Message R_F_S to RTIG");
+
+    comm->sendMessage(&req);
+
+    G.Out(pdGendoc,"exit  FederationManagement::requestFederationSave "
+                   "without time");
+}
+
 // ----------------------------------------------------------------------------
 void
 FederationManagement::federateSaveBegun(TypeException &)
@@ -584,6 +768,7 @@ FederationManagement::federateSaveStatus(bool status, TypeException &)
 void
 FederationManagement::initiateFederateSave(const char *label)
 {
+    G.Out(pdGendoc,"enter FederationManagement::initiateFederateSave");
     D.Out(pdInit, "Initiate a federate save \"%s\".", label);
 
     savingState = true ;
@@ -596,6 +781,8 @@ FederationManagement::initiateFederateSave(const char *label)
     req.setLabel(label);
 
     comm->requestFederateService(&req, &rep);
+
+    G.Out(pdGendoc,"exit  FederationManagement::initiateFederateSave");
 }
 
 // ----------------------------------------------------------------------------
@@ -743,4 +930,4 @@ FederationManagement::checkFederationRestoring()
 
 }} // namespace certi/rtia
 
-// $Id: FederationManagement.cc,v 3.22 2007/04/24 15:10:31 erk Exp $
+// $Id: FederationManagement.cc,v 3.36 2007/11/19 10:20:53 erk Exp $
