@@ -18,21 +18,27 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: ObjectManagement.cc,v 3.19 2007/02/21 10:21:15 rousse Exp $
+// $Id: ObjectManagement.cc,v 3.30 2007/11/16 15:45:01 erk Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
-#include "ObjectManagement.hh"
 
+#include "InteractionSet.hh"
+#include "ObjectSet.hh"
+#include "ObjectClassSet.hh"
+#include "ObjectManagement.hh"
 #include "FederationManagement.hh"
 #include "PrettyDebug.hh"
 
+#ifdef _WIN32
+#include <windows.h>
+#else
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <iostream>
-
+#endif
 using std::cout ;
 using std::endl ;
 using std::string ;
@@ -41,6 +47,7 @@ namespace certi {
 namespace rtia {
 
 static pdCDebug D("RTIA_OM", __FILE__);
+static PrettyDebug G("GENDOC",__FILE__);
 
 ObjectManagement::ObjectManagement(Communications *GC,
                                    FederationManagement *GF,
@@ -85,6 +92,15 @@ ObjectManagement::registerObject(ObjectClassHandle the_class,
 
 // ----------------------------------------------------------------------------
 //! updateAttributeValues
+/** with time
+    @param theObjectHandle Object handle
+    @param attribArray attribute handles array (pointer)
+    @param valueArray value array (pointer)
+    @param attribArraySize attribute and value array size
+    @param theTime time of the federation
+    @param theTag user tag (pointer)
+    @param e exception address (may be modified)
+*/
 EventRetractionHandle
 ObjectManagement::updateAttributeValues(ObjectHandle theObjectHandle,
                                         AttributeHandle *attribArray,
@@ -97,12 +113,14 @@ ObjectManagement::updateAttributeValues(ObjectHandle theObjectHandle,
     NetworkMessage req, rep ;
     int i ;
 
-    // Mise en place de la requete
+    // Building request (req NetworkMessage)
     req.type = NetworkMessage::UPDATE_ATTRIBUTE_VALUES ;
     req.federation = fm->_numero_federation ;
     req.federate = fm->federate ;
     req.object = theObjectHandle ;
     req.date = theTime ;
+    // true for UAV with time
+    req.setBoolean(true);
 
     req.handleArraySize = attribArraySize ;
 
@@ -121,6 +139,50 @@ ObjectManagement::updateAttributeValues(ObjectHandle theObjectHandle,
     return rep.eventRetraction ;
 }
 
+// ----------------------------------------------------------------------------
+//! updateAttributeValues
+/** without time
+    @param theObjectHandle Object handle
+    @param attribArray attribute handles array (pointer)
+    @param valueArray value array (pointer)
+    @param attribArraySize attribute and value array size
+    @param theTag user tag (pointer)
+    @param e exception address (may be modified)
+*/
+void
+ObjectManagement::updateAttributeValues(ObjectHandle theObjectHandle,
+                                        AttributeHandle *attribArray,
+                                        ValueLengthPair *valueArray,
+                                        UShort attribArraySize,
+                                        const char *theTag,
+                                        TypeException &e)
+{
+    NetworkMessage req, rep ;
+    int i ;
+
+    // Building request (req NetworkMessage)
+    req.type = NetworkMessage::UPDATE_ATTRIBUTE_VALUES ;
+    req.federation = fm->_numero_federation ;
+    req.federate = fm->federate ;
+    req.object = theObjectHandle ;
+    // false for UAV without time
+    req.setBoolean(false);
+
+    req.handleArraySize = attribArraySize ;
+
+    for (i = 0 ; i < attribArraySize ; i++) {
+        req.handleArray[i] = attribArray[i] ;
+        req.setValue(i, valueArray[i].value, valueArray[i].length);
+    }
+
+    strcpy(req.label, theTag);
+
+    comm->sendMessage(&req);
+    comm->waitMessage(&rep, NetworkMessage::UPDATE_ATTRIBUTE_VALUES, req.federate);
+
+    e = rep.exception ;
+
+}
 // ----------------------------------------------------------------------------
 //! discoverObject.
 void
@@ -148,7 +210,7 @@ ObjectManagement::discoverObject(ObjectHandle the_object,
 }
 
 // ----------------------------------------------------------------------------
-//! reflectAttributeValues.
+//! reflectAttributeValues with time
 void
 ObjectManagement::reflectAttributeValues(ObjectHandle the_object,
                                          AttributeHandle *the_attributes,
@@ -167,12 +229,36 @@ ObjectManagement::reflectAttributeValues(ObjectHandle the_object,
     req.setEventRetraction(the_event);
     req.setTag(the_tag);
     req.setAttributes(the_attributes, the_values, the_size);
+    // true for RAV without time
+    req.setBoolean(true);
 
     comm->requestFederateService(&req, &rep);
 }
 
 // ----------------------------------------------------------------------------
-//! sendInteraction
+//! reflectAttributeValues without time
+void
+ObjectManagement::reflectAttributeValues(ObjectHandle the_object,
+                                         AttributeHandle *the_attributes,
+                                         ValueLengthPair *the_values,
+                                         UShort the_size,
+                                         const char *the_tag,
+                                         TypeException &)
+{
+    Message req, rep ;
+
+    req.type = Message::REFLECT_ATTRIBUTE_VALUES ;
+    req.setObject(the_object);
+    req.setTag(the_tag);
+    req.setAttributes(the_attributes, the_values, the_size);
+    // false for RAV without time
+    req.setBoolean(false);
+
+    comm->requestFederateService(&req, &rep);
+}
+
+// ----------------------------------------------------------------------------
+//! sendInteraction with time
 EventRetractionHandle
 ObjectManagement::sendInteraction(InteractionClassHandle theInteraction,
                                   ParameterHandle *paramArray,
@@ -184,16 +270,18 @@ ObjectManagement::sendInteraction(InteractionClassHandle theInteraction,
                                   TypeException &e)
 {
     NetworkMessage req, rep ;
-
-    // Test local pour savoir si l'interaction est correcte.
+    G.Out(pdGendoc,"ObjectManagement::sendInteraction with time") ;
+    // Local test to know if interaction is correct.
     rootObject->Interactions->isReady(fm->federate,
                                       theInteraction,
                                       paramArray,
                                       paramArraySize);
 
-    // Preparation du message au RTI.
+    // Building network message (req) to RTIG.
     req.type = NetworkMessage::SEND_INTERACTION ;
     req.interactionClass = theInteraction ;
+    // true for UAV with time
+    req.setBoolean(true);
     req.date = theTime ;
     req.region = region ;
     req.federation = fm->_numero_federation ;
@@ -208,7 +296,7 @@ ObjectManagement::sendInteraction(InteractionClassHandle theInteraction,
 
     strcpy(req.label, theTag);
 
-    // Emission et attente de la reponse.
+    // Send network message and then wait for answer.
     comm->sendMessage(&req);
     comm->waitMessage(&rep, NetworkMessage::SEND_INTERACTION, req.federate);
 
@@ -218,7 +306,52 @@ ObjectManagement::sendInteraction(InteractionClassHandle theInteraction,
 }
 
 // ----------------------------------------------------------------------------
-//! receiveInteraction
+//! sendInteraction without time
+void
+ObjectManagement::sendInteraction(InteractionClassHandle theInteraction,
+                                  ParameterHandle *paramArray,
+                                  ParameterLengthPair *valueArray,
+                                  UShort paramArraySize,
+                                  const char *theTag,
+				  RegionHandle region,
+                                  TypeException &e)
+{
+    NetworkMessage req, rep ;
+    G.Out(pdGendoc,"ObjectManagement::sendInteraction without time");
+    // Local test to know if interaction is correct.
+    rootObject->Interactions->isReady(fm->federate,
+                                      theInteraction,
+                                      paramArray,
+                                      paramArraySize);
+
+    // Building network message (req) to RTIG.
+    req.type = NetworkMessage::SEND_INTERACTION ;
+    req.interactionClass = theInteraction ;
+    // false for UAV without time
+    req.setBoolean(false);
+    req.region = region ;
+    req.federation = fm->_numero_federation ;
+    req.federate = fm->federate ;
+
+    req.handleArraySize = paramArraySize ;
+
+    for (int i=0 ; i<paramArraySize ; i++) {
+	req.handleArray[i] = paramArray[i] ;
+	req.setValue(i, valueArray[i].value, valueArray[i].length);
+    }
+
+    strcpy(req.label, theTag);
+
+    // Send network message and then wait for answer.
+    comm->sendMessage(&req);
+    comm->waitMessage(&rep, NetworkMessage::SEND_INTERACTION, req.federate);
+
+    e = rep.exception ;
+
+}
+
+// ----------------------------------------------------------------------------
+//! receiveInteraction with time
 void
 ObjectManagement::receiveInteraction(InteractionClassHandle the_interaction,
                                      ParameterHandle *the_parameters,
@@ -237,7 +370,28 @@ ObjectManagement::receiveInteraction(InteractionClassHandle the_interaction,
     req.setEventRetraction(the_event);
     req.setTag(the_tag);
     req.setParameters(the_parameters, the_values, the_size);
+    req.setBoolean(true);
+    // BUG: On fait quoi de la reponse ?
+    comm->requestFederateService(&req, &rep);
+}
 
+// ----------------------------------------------------------------------------
+//! receiveInteraction without time
+void
+ObjectManagement::receiveInteraction(InteractionClassHandle the_interaction,
+                                     ParameterHandle *the_parameters,
+                                     ParameterLengthPair *the_values,
+                                     UShort the_size,
+                                     const char *the_tag,
+                                     TypeException &)
+{
+    Message req, rep ;
+
+    req.type = Message::RECEIVE_INTERACTION ;
+    req.setInteractionClass(the_interaction);
+    req.setTag(the_tag);
+    req.setParameters(the_parameters, the_values, the_size);
+    req.setBoolean(false);
     // BUG: On fait quoi de la reponse ?
     comm->requestFederateService(&req, &rep);
 }
@@ -446,10 +600,8 @@ void
 ObjectManagement::provideAttributeValueUpdate(ObjectHandle,
                                               ValueLengthPair &,
                                               TypeException &)
-{
-    printf("ObjectManagement.cc: "
-           "provideAttributeValueUpdate not implemented\n");
-    throw RTIinternalError("");
+{    
+    throw RTIinternalError("ObjectManagement::provideAttributeValueUpdate not implemented");
 }
 
 
@@ -459,9 +611,8 @@ ObjectManagement::provideAttributeValueUpdate(ObjectHandle,
 
 void ObjectManagement::retract(EventRetractionHandle /*theHandle*/,
                                TypeException & /*e*/)
-{
-    printf("GO.cc: retract not implemented.\n");
-    throw RTIinternalError("");
+{    
+    throw RTIinternalError("ObjectManagement::retract not implemented.");
 }
 
 
@@ -472,9 +623,8 @@ void ObjectManagement::retract(EventRetractionHandle /*theHandle*/,
 void
 ObjectManagement::reflectRetraction(EventRetractionHandle,
                                     TypeException &)
-{
-    printf("GO.cc: reflectRetraction not implemented.\n");
-    throw RTIinternalError("");
+{    
+    throw RTIinternalError("ObjectManagement::reflectRetraction not implemented.");
 }
 
 // ----------------------------------------------------------------------------
@@ -578,4 +728,4 @@ ObjectManagement::getObjectClass(ObjectHandle object)
 
 }} // namespace certi/rtia
 
-// $Id: ObjectManagement.cc,v 3.19 2007/02/21 10:21:15 rousse Exp $
+// $Id: ObjectManagement.cc,v 3.30 2007/11/16 15:45:01 erk Exp $
