@@ -18,13 +18,15 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: RTIA_federate.cc,v 3.34 2007/04/20 08:27:07 rousse Exp $
+// $Id: RTIA_federate.cc,v 3.51 2007/11/16 15:04:21 rousse Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
 #include "RTIA.hh"
 
+
 #include "fed.hh"
+#include "RoutingSpace.hh"
 #include "XmlParser.hh"
 
 using std::string ;
@@ -34,6 +36,7 @@ namespace certi {
 namespace rtia {
 
 static pdCDebug D("RTIA", __FILE__);
+static PrettyDebug G("GENDOC",__FILE__) ;
 
 // ----------------------------------------------------------------------------
 //! Verify that federate is not in saving or restoring state.
@@ -41,6 +44,8 @@ void
 RTIA::saveAndRestoreStatus(Message::Type type)
     throw (SaveInProgress, RestoreInProgress)
 {
+//D.Mes(pdMessage, 'N', type);
+
     switch (type) {
       case Message::RESIGN_FEDERATION_EXECUTION:
       case Message::TICK_REQUEST:
@@ -84,8 +89,10 @@ RTIA::saveAndRestoreStatus(Message::Type type)
 //! Choose federate processing.
 void
 RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
-        throw (CouldNotOpenFED)
+       throw (CouldNotOpenFED,FederationExecutionAlreadyExists,ErrorReadingFED)
 {
+    G.Out(pdGendoc,"enter RTIA::chooseFederateProcessing");
+
     // Verify not in saving or restoring state.
     saveAndRestoreStatus(req->type);
 
@@ -137,7 +144,7 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
           else if  ( !strcasecmp(extension.c_str(),"xml") )
               {
               is_an_xml = true ;
-              D.Out(pdTrace, "Trying to use .fed file");
+              D.Out(pdTrace, "Trying to use .xml file");
               } 
           else 
               throw CouldNotOpenFED("nor .fed nor .xml");
@@ -149,7 +156,8 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
                   {        
 	          int result = certi::fedparser::build(filename.c_str(),
 		    				   rootObject, true);
-	          if (result) throw ErrorReadingFED("invalid .fed");
+std::cout<<"RTIA::chooseFederateProcessing result= "<<result<<std::endl;
+	          if (result != 0 ) throw ErrorReadingFED("invalid .fed");
                   }
               else if ( is_an_xml )
                   {
@@ -178,7 +186,14 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
       case Message::REGISTER_FEDERATION_SYNCHRONIZATION_POINT:
         D.Out(pdTrace, "Receiving Message from Federate, type RequestPause.");
 
-        fm->registerSynchronization(req->getLabel(), req->getTag(), e);
+        // boolean true means with federates set
+        if ( req->getBoolean() )
+            fm->registerSynchronization(req->getLabel(), req->getTag(), 
+                (unsigned short)req->handleArraySize, req->handleArray, e);
+        else
+            fm->registerSynchronization(req->getLabel(), req->getTag(), e);
+        // Don't forget boolean for the answer
+        rep.setBoolean(req->getBoolean()) ;
         break ;
 
       case Message::SYNCHRONIZATION_POINT_ACHIEVED:
@@ -188,10 +203,22 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
         break ;
 
       case Message::REQUEST_FEDERATION_SAVE:
-        D.Out(pdTrace, "Receiving Message from Federate, type"
-              " RequestFederationSave.");
-
-        fm->requestFederationSave(req->getLabel(), req->getFederationTime(), e);
+        // boolean true means with time
+        if ( req->getBoolean() )
+            {
+            D.Out(pdTrace, "Receiving Message from Federate, type"
+                           " RequestFederationSave with time.");
+            fm->requestFederationSave(req->getLabel(),req->getFederationTime(),
+                                      e);
+            }
+        else
+            {
+            D.Out(pdTrace, "Receiving Message from Federate, type"
+                           " RequestFederationSave without time.");
+            fm->requestFederationSave(req->getLabel(),e);
+            }
+        // Don't forget boolean value (req Message) for the answer (rep Message)
+        rep.setBoolean(req->getBoolean()) ;
         break ;
 
       case Message::FEDERATE_SAVE_BEGUN:
@@ -305,13 +332,15 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
       }
 
       case Message::UPDATE_ATTRIBUTE_VALUES: {
-          D.Out(pdTrace,
-                "Receiving Message from Federate, type UpdateAttribValues.");
 
           ValueLengthPair *ValueArray = req->getValueArray();
 
           try {
-              rep.setEventRetraction(
+              if (req->getBoolean() )
+                  {
+                  D.Out(pdTrace,"Receiving Message from Federate, type "
+                                "UpdateAttribValues with TIMESTAMP.");
+                  rep.setEventRetraction(
 		  om->updateAttributeValues(req->getObject(),
                                             req->handleArray,
                                             ValueArray,
@@ -319,6 +348,20 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
                                             req->getFederationTime(),
                                             req->getTag(),
                                             e));
+                  }
+              else
+                  {
+                  D.Out(pdTrace,"Receiving Message from Federate, type "
+                                "UpdateAttribValues without TIMESTAMP.");
+		  om->updateAttributeValues(req->getObject(),
+                                            req->handleArray,
+                                            ValueArray,
+                                            req->handleArraySize,
+                                            req->getTag(),
+                                            e);
+                  }
+              // Don't forget boolean value for the answer
+              rep.setBoolean(req->getBoolean()) ;
               free(ValueArray);
           } catch (Exception *e) {
               free(ValueArray);
@@ -328,14 +371,16 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
         break ;
 
       case Message::SEND_INTERACTION: {
-          D.Out(pdTrace,
-                "Receiving Message from Federate, type SendInteraction.");
 
           ParameterLengthPair *ValueArray = (ParameterLengthPair *) req->getValueArray();
-
+          G.Out(pdGendoc,"S_I into RTIA::chooseFederateProcessing") ;
           try {
-              rep.setEventRetraction(
-                  om->sendInteraction(req->getInteractionClass(),
+              if (req->getBoolean() )
+                  {
+                  D.Out(pdTrace,
+   "Receiving Message from Federate, type SendInteraction with TIMESTAMP.");
+                  rep.setEventRetraction(
+                      om->sendInteraction(req->getInteractionClass(),
                                       req->handleArray,
                                       ValueArray,
                                       req->handleArraySize,
@@ -343,6 +388,19 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
                                       req->getTag(),
 				      req->getRegion(),
                                       e));
+                  }
+            else
+                  {
+                  D.Out(pdTrace,
+   "Receiving Message from Federate, type SendInteraction without TIMESTAMP.");
+                      om->sendInteraction(req->getInteractionClass(),
+                                      req->handleArray,
+                                      ValueArray,
+                                      req->handleArraySize,
+                                      req->getTag(),
+				      req->getRegion(),
+                                      e);
+                  }
               free(ValueArray);
           } catch (Exception *e) {
               free(ValueArray);
@@ -522,6 +580,25 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
         rep.setFederationTime(tm->requestLBTS());
         break ;
 
+      case Message::ENABLE_ASYNCHRONOUS_DELIVERY:
+        D.Out(pdTrace, "Receiving Message from Federate, "
+                       "type EnableAsynchronousDelivery.");
+        if (! tm->_asynchronous_delivery)
+           tm->_asynchronous_delivery = true ;
+        else
+           e = e_AsynchronousDeliveryAlreadyEnabled;
+        break ;
+
+      case Message::DISABLE_ASYNCHRONOUS_DELIVERY:
+        D.Out(pdTrace, "Receiving Message from Federate, "
+                       "type DisableAsynchronousDelivery.");
+        if (tm->_asynchronous_delivery)
+           tm->_asynchronous_delivery = false ;
+        else
+          e = e_AsynchronousDeliveryAlreadyDisabled;
+        break ;
+
+
       case Message::QUERY_FEDERATE_TIME:
         D.Out(pdTrace,
               "Receiving Message from Federate, type RequestFederateTime.");
@@ -771,9 +848,17 @@ RTIA::chooseFederateProcessing(Message *req, Message &rep, TypeException &e)
         break ;
 
       case Message::TICK_REQUEST:
-        D.Out(pdDebug, "Receiving Message from Federate, type TickRequest.");
-
+        if (req->getBoolean()) {
+           tm->_ongoing_tick = true ;
+           D.Out(pdDebug, "Receiving Message from Federate, type TickRequest2.");
+           
+        }
+        else {
+           D.Out(pdDebug, "Receiving Message from Federate, type TickRequest.");
+           
+        }
         rep.setBoolean(tm->tick(e));
+
         break ;
 
       default:
@@ -791,6 +876,8 @@ RTIA::processFederateRequest(Message *req)
 {
     Message rep ;
 
+    G.Out(pdGendoc,"enter RTIA::processFederateRequest");
+
     //rep.setException(e_NO_EXCEPTION);
     rep.type = req->type ;
 
@@ -857,7 +944,7 @@ RTIA::processFederateRequest(Message *req)
     }
     catch (CouldNotOpenFED &e) {
         D.Out(pdExcept, "Catched %s Exception.", e._name);
-        rep.setException(e_CouldNotOpenFED);
+        rep.setException(e_CouldNotOpenFED,e._reason);
     }
     catch (CouldNotRestore &e) {
         D.Out(pdExcept, "Catched %s Exception.", e._name);
@@ -869,7 +956,11 @@ RTIA::processFederateRequest(Message *req)
     }
     catch (ErrorReadingRID &e) {
         D.Out(pdExcept, "Catched %s Exception.", e._name);
-        rep.setException(e_ErrorReadingRID);
+        rep.setException(e_ErrorReadingRID,e._reason);
+    }
+    catch (ErrorReadingFED &e) {
+        D.Out(pdExcept, "Catched %s Exception.", e._name);
+        rep.setException(e_ErrorReadingFED,e._reason);
     }
     catch (EventNotKnown &e) {
         D.Out(pdExcept, "Catched %s Exception.", e._name);
@@ -929,7 +1020,7 @@ RTIA::processFederateRequest(Message *req)
     }
     catch (FederationExecutionAlreadyExists &e) {
         D.Out(pdExcept, "Catched %s Exception.", e._name);
-        rep.setException(e_FederationExecutionAlreadyExists);
+        rep.setException(e_FederationExecutionAlreadyExists,e._reason);
     }
     catch (FederationExecutionDoesNotExist &e) {
         D.Out(pdExcept, "Catched %s Exception.", e._name);
@@ -1125,10 +1216,14 @@ RTIA::processFederateRequest(Message *req)
 
     delete req ;
 
-    comm->sendUN(&rep);
-    D.Out(pdDebug, "Reply send to Unix socket.");
+    if (!tm->_ongoing_tick) {
+        comm->sendUN(&rep);
+        D.Out(pdDebug, "Reply send to Unix socket.");
+    }
+    // else, this answer is differed until a no empty tick
+
 }
 
 }} // namespace certi/rtia
 
-// $Id: RTIA_federate.cc,v 3.34 2007/04/20 08:27:07 rousse Exp $
+// $Id: RTIA_federate.cc,v 3.51 2007/11/16 15:04:21 rousse Exp $
