@@ -18,12 +18,16 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: RTIA_network.cc,v 3.10 2007/04/20 12:32:52 erk Exp $
+// $Id: RTIA_network.cc,v 3.13 2007/10/31 10:30:23 erk Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
 #include "RTIA.hh"
 #include "ObjectClassAttribute.hh"
+#include "Interaction.hh"
+#include "InteractionSet.hh"
+#include "ObjectClass.hh"
+#include "ObjectClassSet.hh"
 
 namespace certi {
 namespace rtia {
@@ -35,7 +39,10 @@ static pdCDebug D("RTIA", "(RTIA net) ");
 void
 RTIA::processNetworkMessage(NetworkMessage *msg)
 {
-  NetworkMessage::Type msgType = msg->type;
+NetworkMessage::Type msgType = msg->type;
+
+//D.Mes(pdMessage, 'N', msgType);
+msg->trace("RTIA::processNetworkMessage ");
 
     switch(msgType) {
 
@@ -112,18 +119,27 @@ RTIA::processNetworkMessage(NetworkMessage *msg)
          // At this point of development, we cannot assume this facility and
          // decided to store message as TSO only if all attributes meets TSO
          // criteria. Otherwise, a single message will be enqueue in FIFO.
-         
-         // Retrieve order type
-         updateOrder = TIMESTAMP;
-         for (UShort i=0; i< msg->handleArraySize; ++i)
-         {
-            if (rootObject->ObjectClasses->getWithHandle( msg->objectClass )
-                ->getAttribute(msg->handleArray[i])->order != TIMESTAMP)
-            {
-               updateOrder = RECEIVE;
-               break;
-            }
-         }
+ 
+         // Here we have to consider RAV without time
+         if ( !msg->getBoolean() )
+             {
+             // without time
+             updateOrder = RECEIVE ;
+             }
+         else
+             {        
+             // Retrieve order type
+             updateOrder = TIMESTAMP;
+             for (UShort i=0; i< msg->handleArraySize; ++i)
+               {
+                if (rootObject->ObjectClasses->getWithHandle( msg->objectClass )
+                    ->getAttribute(msg->handleArray[i])->order != TIMESTAMP)
+                {
+                   updateOrder = RECEIVE;
+                   break;
+                }
+               }
+             }
          
          // Decide which queue will be used
          if(updateOrder == TIMESTAMP && tm->requestContraintState())    
@@ -304,4 +320,4 @@ RTIA::processNetworkMessage(NetworkMessage *msg)
 
 }} // namespace certi/rtia
 
-// $Id: RTIA_network.cc,v 3.10 2007/04/20 12:32:52 erk Exp $
+// $Id: RTIA_network.cc,v 3.13 2007/10/31 10:30:23 erk Exp $
