@@ -18,7 +18,7 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: TimeManagement.cc,v 3.15 2007/02/21 10:21:15 rousse Exp $
+// $Id: TimeManagement.cc,v 3.22 2007/11/12 14:49:10 siron Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -78,6 +78,8 @@ TimeManagement::TimeManagement(Communications *GC,
     lastNullMessageDate = 0.0 ;
 
     _avancee_en_cours = PAS_D_AVANCEE ;
+    _ongoing_tick          = false ;
+    _asynchronous_delivery = false ;
 
     _heure_courante = 0.0 ;
     _lookahead_courant = epsilon ;
@@ -115,7 +117,11 @@ void TimeManagement::sendNullMessage(FederationTime heure_logique)
 bool
 TimeManagement::executeFederateService(NetworkMessage &msg)
 {
-    D.Out(pdRequest, "Execute federate service: Type %d.", msg.type);
+  D.Out(pdRequest, "Execute federate service: Type %d.", msg.type);
+
+  msg.trace("TimeManagement::executeFederateService ");
+
+  _ongoing_tick = false ;  // end of the blocking tick, a message is delivered
 
     switch (msg.type) {
 
@@ -170,14 +176,22 @@ TimeManagement::executeFederateService(NetworkMessage &msg)
       {
           ValueLengthPair *ValueArray = msg.getAttribValueArray();
 
-          om->reflectAttributeValues(msg.object,
-                                     msg.handleArray,
-                                     ValueArray,
-                                     msg.handleArraySize,
-                                     msg.date,
-                                     msg.label,
-                                     msg.eventRetraction,
-                                     msg.exception);
+          if (msg.getBoolean())
+             om->reflectAttributeValues(msg.object,
+                                        msg.handleArray,
+                                        ValueArray,
+                                        msg.handleArraySize,
+                                        msg.date,
+                                        msg.label,
+                                        msg.eventRetraction,
+                                        msg.exception);
+          else
+            om->reflectAttributeValues(msg.object,
+                                        msg.handleArray,
+                                        ValueArray,
+                                        msg.handleArraySize,
+                                        msg.label,
+                                        msg.exception);
           free(ValueArray);
           break ;
       }
@@ -186,14 +200,22 @@ TimeManagement::executeFederateService(NetworkMessage &msg)
       {
           ParameterLengthPair *ValueArray = msg.getParamValueArray();
 
-          om->receiveInteraction(msg.interactionClass,
-                                 msg.handleArray,
-                                 ValueArray,
-                                 msg.handleArraySize,
-                                 msg.date,
-                                 msg.label,
-                                 msg.eventRetraction,
-                                 msg.exception);
+          if (msg.getBoolean())
+              om->receiveInteraction(msg.interactionClass,
+                                     msg.handleArray,
+                                     ValueArray,
+                                     msg.handleArraySize,
+                                     msg.date,
+                                     msg.label,
+                                     msg.eventRetraction,
+                                     msg.exception);
+          else
+              om->receiveInteraction(msg.interactionClass,
+                                     msg.handleArray,
+                                     ValueArray,
+                                     msg.handleArraySize,
+                                     msg.label,
+                                     msg.exception);
           free(ValueArray);
 
           break ;
@@ -577,7 +599,10 @@ TimeManagement::tick(TypeException &e)
 
     // 2nd try, give a FIFO message. (discoverObject, etc.)
     if (!msg_donne)
-        msg = queues->giveFifoMessage(msg_donne, msg_restant);
+        if ( _asynchronous_delivery || (_avancee_en_cours != PAS_D_AVANCEE))
+          // to exclude the case not asynchronous_delivery and
+          // not time advancing
+          msg = queues->giveFifoMessage(msg_donne, msg_restant);
 
     // If message exists, send it to federate.
     if (msg_donne) {
@@ -675,6 +700,8 @@ TimeManagement::timeAdvanceGrant(FederationTime logical_time,
     D.Out(pdRegister, "timeAdvanceGrant sent to federate (time = %f).",
           req.getFederationTime());
 
+    _ongoing_tick = false ;  // end of the blocking tick, a message is delivered
+
     comm->requestFederateService(&req, &rep);
 
     e = rep.getExceptionType();
@@ -720,4 +747,4 @@ TimeManagement::timeAdvanceRequest(FederationTime logical_time,
 
 }} // namespaces
 
-// $Id: TimeManagement.cc,v 3.15 2007/02/21 10:21:15 rousse Exp $
+// $Id: TimeManagement.cc,v 3.22 2007/11/12 14:49:10 siron Exp $
