@@ -18,18 +18,36 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: Federation.cc,v 3.52 2007/05/03 15:46:31 rousse Exp $
+// $Id: Federation.cc,v 3.69 2007/11/16 15:04:22 rousse Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
 #include "Federation.hh"
+#include <sstream>
+
+#ifdef _WIN32
+
+#endif
 
 #include "fed.hh"
 #include "XmlParser.hh"
+#include "Object.hh"
+#include "ObjectAttribute.hh"
+#include "ObjectSet.hh"
+#include "ObjectClassSet.hh"
+#include "InteractionSet.hh"
+
 #include "ObjectClassAttribute.hh"
 #include "PrettyDebug.hh"
 #include "LBTS.hh"
 
+#ifdef _WIN32
+#include <windows.h>
+#include <time.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <stdio.h>
+#else
 #include <map>
 #include <fstream>
 #include <iostream>
@@ -38,6 +56,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <errno.h>
+#endif
 
 using std::pair ;
 using std::ifstream ;
@@ -63,6 +82,7 @@ class XmlParser ;
 namespace rtig {
 
 static PrettyDebug D("FEDERATION", __FILE__);
+static PrettyDebug G("GENDOC",__FILE__);
 
 // ----------------------------------------------------------------------------
 //! Constructor
@@ -70,19 +90,33 @@ static PrettyDebug D("FEDERATION", __FILE__);
   result in RootObj.
 */
 #ifdef FEDERATION_USES_MULTICAST
+/** with FEDERATION_USES_MULTICAST defined
+    @param federation_name
+    @param federation_handle
+    @param socket_server
+    @param audit_server
+    @param mc_link
+*/
 Federation::Federation(const char *federation_name,
                        FederationHandle federation_handle,
                        SocketServer &socket_server,
                        AuditFile &audit_server,
                        SocketMC *mc_link)
 #else
+/** with FEDERATION_USES_MULTICAST not defined
+    @param federation_name
+    @param federation_handle
+    @param socket_server
+    @param audit_server
+    @param FEDid_name i.e. FED file name (may be a .fed or a .xml file)
+*/
     Federation::Federation(const char *federation_name,
                            Handle federation_handle,
                            SocketServer &socket_server,
                            AuditFile &audit_server,
                            const char *FEDid_name)
 #endif
-    throw (CouldNotOpenFED, ErrorReadingRID, MemoryExhausted, SecurityError,
+    throw (CouldNotOpenFED, ErrorReadingFED, MemoryExhausted, SecurityError,
            RTIinternalError)
     : federateHandles(1), objectHandles(1), saveInProgress(false),
       restoreInProgress(false), saveStatus(true), restoreStatus(true),
@@ -90,7 +124,7 @@ Federation::Federation(const char *federation_name,
 
 {
     //    fedparser::FedParser *fed_reader ;
-  struct stat file_stat;
+  STAT_STRUCT file_stat;
 
 #ifdef FEDERATION_USES_MULTICAST // -----------------
     // Initialize Multicast
@@ -110,6 +144,8 @@ Federation::Federation(const char *federation_name,
     if (strlen(federation_name) > MAX_FEDERATION_NAME_LENGTH)
         throw RTIinternalError("Federation name too long.");
 
+    G.Out(pdGendoc,"enter Federation::Federation");
+
     name = strdup(federation_name);
 
     // Default Attribute values
@@ -144,25 +180,46 @@ Federation::Federation(const char *federation_name,
     cout << "Looking for FOM file... " << endl ; 
 
     cout << "   Trying... " << filename;
-    filefound = (0==stat(filename.c_str(),&file_stat));
+    filefound = (0==STAT_FUNCTION(filename.c_str(),&file_stat));
 
+#ifdef _WIN32
+    if (!filefound) {
+      char temp[260];
+      cout << " --> cannot access." <<endl;
+      GetCurrentDirectory(260,temp);
+      filename = string(temp);
+      filename = filename + "\\share\\federations\\"+string(FEDid_name);
+      cout << "   Now trying..." << filename;
+      filefound = (0==STAT_FUNCTION(filename.c_str(),&file_stat));
+    }
+    
+    if (!filefound && (NULL!=getenv("CERTI_HOME"))) {
+      cout << " --> cannot access." <<endl;
+      filename = string(getenv("CERTI_HOME"))+"\\share\\federations\\"+FEDid_name;
+      cout << "   Now trying..." << filename;      
+      filefound = (0==STAT_FUNCTION(filename.c_str(),&file_stat));
+    }
+#else
     if (!filefound) {
       cout << " --> cannot access." <<endl;
-      filename = "/usr/local/share/federation/"+string(FEDid_name);
+      filename = "/usr/local/share/federations/"+string(FEDid_name);
       cout << "   Now trying..." << filename;
+      filefound = (0==STAT_FUNCTION(filename.c_str(),&file_stat));
     }
-    filefound = (0==stat(filename.c_str(),&file_stat));
-
-    if (!filefound) {
+    
+    if (!filefound && (NULL!=getenv("CERTI_HOME"))) {
       cout << " --> cannot access." <<endl;
       filename = string(getenv("CERTI_HOME"))+"/share/federations/"+FEDid_name;
-      cout << "   Now trying..." << filename;
+      cout << "   Now trying..." << filename;      
+      filefound = (0==STAT_FUNCTION(filename.c_str(),&file_stat));
     }
-    filefound = (0==stat(filename.c_str(),&file_stat));
+#endif
     
     if (!filefound) {
       cout << " --> cannot access." <<endl;
       cerr << "Next step will fail"<<endl;
+      G.Out(pdGendoc,"exit Federation::Federation on exception CouldNotOpenFED");
+      throw CouldNotOpenFED("RTIG cannot find FED file.");
     }
 
     // now really assign FEDid
@@ -174,7 +231,8 @@ Federation::Federation(const char *federation_name,
     if ( (fftry=fopen(FEDid,"r")) == NULL)
         {
         cout << "... failed : ";
-        throw CouldNotOpenFED("FED file unknown.");
+        G.Out(pdGendoc,"exit Federation::Federation on exception CouldNotOpenFED");
+        throw CouldNotOpenFED("RTIG have found but cannot open FED file");
         }
     else
         {
@@ -187,8 +245,12 @@ Federation::Federation(const char *federation_name,
     bool is_an_xml      = false ;
     
     // hope there is a . before fed or xml
-    if ( filename.at(nbcar_filename-4) != '.' )
-        throw CouldNotOpenFED("FED file incorrect filename, cannot find extension (character '.' is missing [or not in reverse 4th place])");
+    if ( filename[nbcar_filename-4] != '.' )
+        {
+        G.Out(pdGendoc,"exit Federation::Federation on exception CouldNotOpenFED");
+        throw CouldNotOpenFED("Incorrect FED file name, cannot find "
+         "extension (character '.' is missing [or not in reverse 4th place])");
+        }       
 
     string extension = filename.substr(nbcar_filename-3,3) ;
     D.Out(pdTrace,"filename is: %s (extension is <%s>",filename.c_str(),extension.c_str());
@@ -202,25 +264,39 @@ Federation::Federation(const char *federation_name,
         is_an_xml = true ;
         D.Out(pdTrace, "Trying to use .xml file");
         } 
-    else 
-        throw CouldNotOpenFED("FED file incorrect filename : nor .fed nor .xml file");
+    else {
+        G.Out(pdGendoc,"exit Federation::Federation on exception CouldNotOpenFED");
+        throw CouldNotOpenFED("Incorrect FED file name : nor .fed nor .xml file");
+    }
        
     ifstream fdd(filename.c_str());
 
     if (fdd.is_open())
         {
-	fdd.close();
+    	fdd.close();
         if ( is_a_fed )
             {
 	    int err = fedparser::build(filename.c_str(), root, verbose);
-	    if (err) throw ErrorReadingFED("");
-	    
-            // Retrieve the FED file last modification time(for Audit)
-            struct stat StatBuffer ;
-            char *MTimeBuffer ;
+	    if (err != 0 ) 
+                {
+                G.Out(pdGendoc,"exit Federation::Federation on exception ErrorReadingFED");                
+                throw ErrorReadingFED("fed parser found error in FED file");
+	        }
 
-            if (stat(filename.c_str(), &StatBuffer) == 0) {
+            // Retrieve the FED file last modification time(for Audit)
+            STAT_STRUCT StatBuffer ;
+            #if defined(_WIN32) && not defined(__MINGW32__)
+            char MTimeBuffer[26];
+            #else
+            char *MTimeBuffer ;
+            #endif
+
+            if (STAT_FUNCTION(filename.c_str(), &StatBuffer) == 0) {
+            #if defined(_WIN32) && not defined(__MINGW32__)                
+                ctime_s(&MTimeBuffer[0],26,&StatBuffer.st_mtime);
+            #else
                 MTimeBuffer = ctime(&StatBuffer.st_mtime);
+            #endif    
                 MTimeBuffer[strlen(MTimeBuffer) - 1] = 0 ; // Remove trailing \n
                 server->audit << "(Last modified " << MTimeBuffer << ")" ;
             }
@@ -249,10 +325,14 @@ Federation::Federation(const char *federation_name,
             }
             else {
                 cout << "nor fed nor xml" << endl ;
-		throw CouldNotOpenFED("");
+                G.Out(pdGendoc,"exit Federation::Federation on exception CouldNotOpenFED");
+		throw CouldNotOpenFED("Incorrect FED file name : nor fed nor xml");
 	    }
         }
     }
+
+    G.Out(pdGendoc,"exit Federation::Federation");
+
 }
 
 // ----------------------------------------------------------------------------
@@ -494,8 +574,61 @@ Federation::broadcastAnyMessage(NetworkMessage *msg,
 }
 
 // ----------------------------------------------------------------------------
-//! broadcastInteraction
+//! Broadcast 'msg' to some Federates except the specified one
+void
+Federation::broadcastSomeMessage(NetworkMessage *msg,
+                                FederateHandle except_federate,
+                                FederateHandle *fede_array,
+                                int nbfed)
+{
+    int ifed ;
+    Socket *socket = NULL ;
 
+    if ( fede_array != NULL || nbfed == 0)
+        {
+        // Broadcast the message 'msg' to some Federates (done in fede_array)
+        // in the Federation
+        // except to Federate whose Handle is 'Except_Federate'.
+        for (FederateList::const_iterator i = federates.begin(); i != federates.end(); ++i)
+            {
+            if (i->getHandle() != except_federate)
+                {
+                ifed = 0 ;
+                while ( ifed < nbfed )
+                    {
+                    if ( i->getHandle() == fede_array[ifed] )
+                        // Federate i has to be informed because into fede_array
+                        {
+                        try
+                            {
+#ifdef HLA_USES_UDP
+                            socket = server->getSocketLink(i->getHandle(), BEST_EFFORT);
+#else
+                            socket = server->getSocketLink(i->getHandle());
+#endif
+                            msg->write(socket);
+                            }
+                        catch (RTIinternalError &e)
+                            {
+                            D[pdExcept] << "Reference to a killed Federate while "
+                                        << "broadcasting." << endl ;
+                            }
+                        catch (NetworkError &e)
+                            {
+                            D.Out(pdExcept, "Network error while broadcasting, ignoring.");
+                            }
+                        }
+                    ifed++;
+                    } 
+                }
+            }
+        }
+
+    // BUG: If except = 0, could use Multicast.
+}
+
+// ----------------------------------------------------------------------------
+//! broadcastInteraction with time
 void
 Federation::broadcastInteraction(FederateHandle federate_handle,
                                  InteractionClassHandle interaction,
@@ -513,6 +646,9 @@ Federation::broadcastInteraction(FederateHandle federate_handle,
            RestoreInProgress,
            RTIinternalError)
 {
+
+    G.Out(pdGendoc,"enter Federation::broadcastInteraction with time");
+
     // It may throw FederateNotExecutionMember.
     this->check(federate_handle);
 
@@ -535,6 +671,56 @@ Federation::broadcastInteraction(FederateHandle federate_handle,
               " Param %d Value %s",
               parameter_handles[i],
               parameter_values[i]);
+
+    G.Out(pdGendoc,"exit Federation::broadcastInteraction with time");
+
+}
+
+// ----------------------------------------------------------------------------
+//! broadcastInteraction without time
+void
+Federation::broadcastInteraction(FederateHandle federate_handle,
+                                 InteractionClassHandle interaction,
+                                 ParameterHandle *parameter_handles,
+                                 ParameterLengthPair *parameter_values,
+                                 UShort list_size,
+				 RegionHandle region_handle,
+                                 const char *tag)
+    throw (FederateNotExecutionMember,
+           FederateNotPublishing,
+           InteractionClassNotDefined,
+           InteractionParameterNotDefined,
+           SaveInProgress,
+           RestoreInProgress,
+           RTIinternalError)
+{
+
+    G.Out(pdGendoc,"enter Federation::broadcastInteraction without time");
+
+    // It may throw FederateNotExecutionMember.
+    this->check(federate_handle);
+
+    const RTIRegion *region = 0 ;
+    if (region_handle != 0)
+	region = root->getRegion(region_handle);
+
+    root->Interactions->broadcastInteraction(federate_handle,
+                                             interaction,
+                                             parameter_handles,
+                                             parameter_values,
+                                             list_size,
+					     region,
+                                             tag);
+    D.Out(pdRequest, "Federation %d: Broadcasted Interaction %d from Federate "
+          "%d nb params %d.", handle, interaction, federate_handle, list_size);
+    for (int i=0 ; i < list_size ; i++)
+        D.Out(pdRequest,
+              " Param %d Value %s",
+              parameter_handles[i],
+              parameter_values[i]);
+
+    G.Out(pdGendoc,"exit Federation::broadcastInteraction without time");
+
 }
 
 // ----------------------------------------------------------------------------
@@ -576,6 +762,9 @@ Federation::registerSynchronization(FederateHandle federate,
            RestoreInProgress,
            RTIinternalError)
 {
+
+    G.Out(pdGendoc,"enter Federation::registerSynchronization for all federates");
+
     this->check(federate); // It may throw FederateNotExecutionMember.
 
     if ((label == NULL) || (strlen(label) > MAX_USER_TAG_LENGTH))
@@ -602,6 +791,64 @@ Federation::registerSynchronization(FederateHandle federate,
 
     D[pdTerm] << "Federation " << handle << " is now synchronizing for label "
               << label << endl ;
+
+    G.Out(pdGendoc,"exit  Federation::registerSynchronization for all federates");
+
+
+}
+
+// ----------------------------------------------------------------------------
+//! Add a new synchronization point (with federates set) to federation.
+void
+Federation::registerSynchronization(FederateHandle federate,
+                                    const char *label,
+                                    const char *tag,
+                                    unsigned short federate_setSize,
+                                    FederateHandle *federate_set)
+    throw (FederateNotExecutionMember,
+           FederationAlreadyPaused,
+           SaveInProgress,
+           RestoreInProgress,
+           RTIinternalError)
+{
+
+    G.Out(pdGendoc,"enter Federation::registerSynchronization for federate set");
+
+    this->check(federate); // It may throw FederateNotExecutionMember.
+
+    if ((label == NULL) || (strlen(label) > MAX_USER_TAG_LENGTH))
+        throw RTIinternalError("Bad pause label(null or too long).");
+
+    // Verify label does not already exists
+    std::map<const char *, const char *>::const_iterator i ;
+    i = synchronizationLabels.begin();
+    for (; i != synchronizationLabels.end(); i++) {
+        if (!strcmp((*i).first, label)) {
+            throw FederationAlreadyPaused(""); // Label already pending.
+        }
+    }
+
+    // If not already in pending labels, insert to list.
+    synchronizationLabels.insert(pair<const char *, const char *>(strdup(label),
+                                                                  strdup(tag)));
+
+    // Add label to each federate into the set only (may throw RTIinternalError).
+    FederateList::iterator j ;
+    for (int i=0 ; i < federate_setSize  ;i++ )
+        {
+        for (j = federates.begin(); j != federates.end(); ++j)
+            {
+            if ( (federate_set[i] == j->getHandle()) || (federate == j->getHandle()) ) 
+                             j->addSynchronizationLabel(label);
+            }
+        }
+
+    D[pdTerm] << "Federation " << handle << " is now synchronizing for label "
+              << label << endl ;
+
+    G.Out(pdGendoc,"exit  Federation::registerSynchronization for federate set");
+
+
 }
 
 // ----------------------------------------------------------------------------
@@ -614,6 +861,9 @@ Federation::broadcastSynchronization(FederateHandle federate,
                                      const char *tag)
     throw (RTIinternalError)
 {
+
+    G.Out(pdGendoc,"enter Federation::broadcastSynchronization");
+
     this->check(federate); // It may throw FederateNotExecutionMember.
 
     if ((label == NULL) || (strlen(label) > MAX_USER_TAG_LENGTH))
@@ -627,20 +877,64 @@ Federation::broadcastSynchronization(FederateHandle federate,
     msg.setLabel(label);
     msg.setTag(tag);
 
+    G.Out(pdGendoc,"      broadcastSynchronization is calling broadcastAnyMessage for all federates");
+
     broadcastAnyMessage(&msg, 0);
+
+    G.Out(pdGendoc,"exit  Federation::broadcastSynchronization");
+
 }
 
 // ----------------------------------------------------------------------------
-//! Request a federation save.
+/*! Broadcast an 'Announce Synchronization Point' when registering a new
+  synchronization point onto a set of federates
+*/
+void
+Federation::broadcastSynchronization(FederateHandle federate,
+                                     const char *label,
+                                     const char *tag,
+                                     unsigned short federate_setSize,
+                                     FederateHandle *federate_set)
+    throw (RTIinternalError)
+{
+
+    G.Out(pdGendoc,"enter Federation::broadcastSynchronization to some federates");
+
+    this->check(federate); // It may throw FederateNotExecutionMember.
+
+    if ((label == NULL) || (strlen(label) > MAX_USER_TAG_LENGTH))
+        throw RTIinternalError("Bad pause label(null or too long).");
+
+    // broadcast announceSynchronizationPoint() to all federates in federation.
+    NetworkMessage msg ;
+    msg.type = NetworkMessage::ANNOUNCE_SYNCHRONIZATION_POINT ;
+    msg.federate = federate ;
+    msg.federation = handle ;
+    msg.setLabel(label);
+    msg.setTag(tag);
+
+    G.Out(pdGendoc,"      broadcastSynchronization is calling broadcastSomeMessage");
+
+    broadcastSomeMessage(&msg, 0, federate_set, (unsigned short)federate_setSize);
+
+    G.Out(pdGendoc,"exit  Federation::broadcastSynchronization to some federates");
+
+}
+
+
+// ----------------------------------------------------------------------------
+//! Request a federation save with time.
 /*! This service puts each federate from federation in saving state.
   !! the_time is not managed yet.
 */
 void
 Federation::requestFederationSave(FederateHandle the_federate,
                                   const char *the_label,
-                                  FederationTime /* the_time */)
+                                  FederationTime time )
     throw (FederateNotExecutionMember, SaveInProgress)
 {
+    G.Out(pdGendoc,"enter Federation::requestFederationSave with time");
+
     check(the_federate);
 
     if (saveInProgress)
@@ -659,10 +953,57 @@ Federation::requestFederationSave(FederateHandle the_federate,
     msg.federate = the_federate ;
     msg.federation = handle ;
     msg.setLabel(the_label);
+    // boolean true means with time and needs time
+    msg.setBoolean(true);
+    msg.date = time;
+
+    G.Out(pdGendoc,"      requestFederationSave====>broadcast I_F_S to all");
 
     broadcastAnyMessage(&msg, 0);
+
+    G.Out(pdGendoc,"exit  Federation::requestFederationSave with time");
 }
 
+// ----------------------------------------------------------------------------
+//! Request a federation save without time.
+/*! This service puts each federate from federation in saving state.
+  !! the_time is not managed yet.
+*/
+void
+Federation::requestFederationSave(FederateHandle the_federate,
+                                  const char *the_label)
+    throw (FederateNotExecutionMember, SaveInProgress)
+{
+    G.Out(pdGendoc,"enter Federation::requestFederationSave without time");
+
+    check(the_federate);
+
+    if (saveInProgress)
+        throw SaveInProgress("Already in saving state.");
+
+    for (FederateList::iterator j = federates.begin(); j != federates.end(); ++j) {
+        j->setSaving(true);
+    }
+
+    saveStatus = true ;
+    saveInProgress = true ;
+    saveLabel = the_label ;
+
+    NetworkMessage msg ;
+    msg.type = NetworkMessage::INITIATE_FEDERATE_SAVE ;
+    msg.federate = the_federate ;
+    msg.federation = handle ;
+    msg.setLabel(the_label);
+    // boolean false means without time
+    msg.setBoolean(false);
+
+    G.Out(pdGendoc,"                  requestFederationSave====>broadcast I_F_S"
+                   " to all");
+   
+    broadcastAnyMessage(&msg, 0);
+
+    G.Out(pdGendoc,"exit  Federation::requestFederationSave without time");
+}
 // ----------------------------------------------------------------------------
 /*! Received from a federate to inform a save has been received and is being
   processed.
@@ -855,12 +1196,15 @@ Federate &
 Federation::getFederate(const char *federate_name)
     throw (FederateNotExecutionMember)
 {
+	std::stringstream msg;
+	
     for (FederateList::iterator i = federates.begin(); i != federates.end(); ++i) {
         if (strcmp(i->getName(), federate_name) == 0)
             return *i ;
     }
 
-    throw FederateNotExecutionMember("Federate Name not found.");
+    msg << "Federate <"<<federate_name<<"> not [yet] member of Federation <" << getName() <<">.";
+    throw FederateNotExecutionMember(msg.str().c_str());
 }
 
 // ----------------------------------------------------------------------------
@@ -1106,6 +1450,9 @@ Federation::unregisterSynchronization(FederateHandle federate_handle,
            RestoreInProgress,
            RTIinternalError)
 {
+
+    G.Out(pdGendoc,"enter Federation::unregisterSynchronization");
+
     this->check(federate_handle); // It may throw FederateNotExecutionMember.
 
     if ((label == NULL) || (strlen(label) > MAX_USER_TAG_LENGTH))
@@ -1148,6 +1495,9 @@ Federation::unregisterSynchronization(FederateHandle federate_handle,
     broadcastAnyMessage(&msg, 0);
 
     D.Out(pdTerm, "Federation %d is synchronized on %s.", handle, label);
+
+    G.Out(pdGendoc,"exit  Federation::unregisterSynchronization");
+
 }
 
 // ----------------------------------------------------------------------------
@@ -1201,7 +1551,7 @@ Federation::subscribeObject(FederateHandle federate,
 }
 
 // ----------------------------------------------------------------------------
-// updateAttributeValues
+// updateAttributeValues with time
 
 void
 Federation::updateAttributeValues(FederateHandle federate,
@@ -1231,6 +1581,35 @@ Federation::updateAttributeValues(FederateHandle federate,
           handle, federate, id);
 }
 
+// ----------------------------------------------------------------------------
+// updateAttributeValues without time
+
+void
+Federation::updateAttributeValues(FederateHandle federate,
+                                  ObjectHandle id,
+                                  AttributeHandle *attributes,
+                                  ValueLengthPair *values,
+                                  UShort list_size,
+                                  const char *tag)
+    throw (FederateNotExecutionMember,
+           ObjectNotKnown,
+           AttributeNotDefined,
+           AttributeNotOwned,
+           SaveInProgress,
+           RestoreInProgress,
+           RTIinternalError)
+{
+    // It may throw FederateNotExecutionMember.
+    this->check(federate);
+
+    // It may throw *NotDefined
+    root->ObjectClasses->updateAttributeValues(federate, id, attributes, values,
+                                               list_size, tag);
+
+    D.Out(pdRegister,
+          "Federation %d: Federate %d updated attributes of Object %d.",
+          handle, federate, id);
+}
 // ----------------------------------------------------------------------------
 //! Update the current time of a regulator federate.
 void
@@ -1732,11 +2111,10 @@ Federation::restoreXmlData()
         return false ;
     }
 
-    char *tmp ;
-    tmp = (char *)xmlGetProp(cur, (const xmlChar*)"name");
-    if (strcmp(name, tmp) != 0) {
+    if (strcmp(name, XmlParser::CleanXmlGetProp(cur,(const xmlChar*)"name")) != 0) {
         cerr << "Wrong federation name" << endl ;
     }
+    
 
     cur = cur->xmlChildrenNode ;
 
@@ -1744,22 +2122,23 @@ Federation::restoreXmlData()
     while (cur != NULL) {
         if ((!xmlStrcmp(cur->name, NODE_FEDERATE))) {
             for (FederateList::iterator i = federates.begin(); i != federates.end(); ++i) {
-                if (!strcmp(i->getName(), (char *) xmlGetProp(cur, (const xmlChar*) "name"))) {
+                if (!strcmp(i->getName(),XmlParser::CleanXmlGetProp(cur, (const xmlChar*) "name"))) {
                     // Set federate constrained status
-                    if (!strcmp("true", (char *) xmlGetProp(cur, (const xmlChar*) "constrained")))
+                    if (!strcmp("true", XmlParser::CleanXmlGetProp(cur, (const xmlChar*) "constrained"))) {
                         status = true ;
-                    else
+                    }
+                    else {
                         status = false ;
+                    }
 
                     i->setConstrained(status);
 
                     // Set federate regulating status
-                    status = !strcmp("true", (char *) xmlGetProp(
-                                         cur, (const xmlChar *) "regulator"));
+                    status = !strcmp("true", XmlParser::CleanXmlGetProp(cur, (const xmlChar *) "regulator"));
 
                     i->setRegulator(status);
 
-                    i->setHandle(strtol((char *) xmlGetProp(cur, (const xmlChar *) "handle"), 0, 10));
+                    i->setHandle(strtol(XmlParser::CleanXmlGetProp(cur, (const xmlChar *) "handle"), 0, 10));
                     break ;
                 }
             }
@@ -1822,5 +2201,5 @@ Federation::saveXmlData()
 
 }} // namespace certi/rtig
 
-// $Id: Federation.cc,v 3.52 2007/05/03 15:46:31 rousse Exp $
+// $Id: Federation.cc,v 3.69 2007/11/16 15:04:22 rousse Exp $
 
