@@ -18,7 +18,7 @@
 // along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: RTIG_processing.cc,v 3.33 2007/04/20 08:27:07 rousse Exp $
+// $Id: RTIG_processing.cc,v 3.44 2007/11/16 15:04:22 rousse Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -33,6 +33,7 @@ namespace certi {
 namespace rtig {
 
 static PrettyDebug D("RTIG", __FILE__);
+static PrettyDebug G("GENDOC",__FILE__);
 
 // ----------------------------------------------------------------------------
 //! Creates a new federation.
@@ -41,8 +42,17 @@ RTIG::processCreateFederation(Socket *link, NetworkMessage *req)
 {
     char *federation = req->federationName ;
     char *FEDid = req->FEDid ;
+    NetworkMessage rep ;               // rep for answer to RTIA
+
+    G.Out(pdGendoc,"BEGIN ** CREATE FEDERATION SERVICE **");
+    G.Out(pdGendoc,"enter RTIG::processCreateFederation");
+
+    if (federation == NULL) 
+        {
+        G.Out(pdGendoc,"exit  RTIG::processCreateFederation on exception RTIinternalError");        
+        throw RTIinternalError("Invalid Federation Name.");
+        }
 
-    if (federation == NULL) throw RTIinternalError("Invalid Federation Name.");
     auditServer << "Federation Name : " << federation ;
     Handle h = federationHandles.provide();
 
@@ -55,6 +65,7 @@ RTIG::processCreateFederation(Socket *link, NetworkMessage *req)
     com_mc = new SocketMC();
     if (com_mc == NULL) {
         D.Out(pdExcept, "Unable to allocate Multicast socket.");
+        G.Out(pdGendoc,"exit  RTIG::processCreateFederation on exception RTIinternalError")
         throw RTIinternalError("Unable to allocate Multicast socket.");
     }
 
@@ -67,20 +78,45 @@ RTIG::processCreateFederation(Socket *link, NetworkMessage *req)
     ClientSockets.push_front(com_mc);
 
 #else
-    federations.createFederation(federation, h, FEDid);
-#endif
-
-    // Prepare answer for RTIA
-    NetworkMessage rep ;
-    rep.type = NetworkMessage::CREATE_FEDERATION_EXECUTION ;
     rep.exception = e_NO_EXCEPTION ;
-    rep.federation = h ;
-    strcpy(rep.FEDid,FEDid) ;
+    // We catch createFederation because it is useful to send
+    // exception reason to RTIA 
+    try {
+        federations.createFederation(federation, h, FEDid);
+        }
+    catch (CouldNotOpenFED e)
+        {
+        rep.exception = e_CouldNotOpenFED ;
+        strcpy(rep.exceptionReason,e._reason) ;
+        }
+    catch (ErrorReadingFED e)
+        {
+        rep.exception = e_ErrorReadingFED ;
+        strcpy(rep.exceptionReason,e._reason) ;
+        }
+    catch (FederationExecutionAlreadyExists e)
+        {
+        rep.exception = e_FederationExecutionAlreadyExists ;
+        strcpy(rep.exceptionReason,e._reason) ;
+        }
+#endif
+    // Prepare answer for RTIA : store NetworkMessage rep
+    rep.type = NetworkMessage::CREATE_FEDERATION_EXECUTION ;
+    if ( rep.exception == e_NO_EXCEPTION )
+        {
+        rep.federation = h ;
+        strcpy(rep.FEDid,FEDid) ;
+        }
+
+    G.Out(pdGendoc,"processCreateFederation===>write");
 
     rep.write(link); // Send answer to RTIA
 
     D.Out(pdInit, "Federation \"%s\" created with Handle %d.",
           federation, rep.federation);
+
+    G.Out(pdGendoc,"exit RTIG::processCreateFederation");
+    G.Out(pdGendoc,"END ** CREATE FEDERATION SERVICE **");
 }
 
 // ----------------------------------------------------------------------------
@@ -102,6 +138,9 @@ RTIG::processJoinFederation(Socket *link, NetworkMessage *req)
     int nb_federes ;
     bool pause ;
 
+    G.Out(pdGendoc,"BEGIN ** JOIN FEDERATION SERVICE **");
+    G.Out(pdGendoc,"enter RTIG::processJoinFederation");
+
     if ((federation == NULL) || (federate == NULL))
         throw RTIinternalError("Invalid Federation/Federate Name.");
 
@@ -168,6 +207,8 @@ RTIG::processJoinFederation(Socket *link, NetworkMessage *req)
     // Send answer
     D.Out(pdTrace,"send NetworkMessage of Type %d after open \"%s\"",
           repFED.type,repFED.FEDid);
+
+    G.Out(pdGendoc,"processJoinFederation====>Begin FED file transfer");
     repFED.write(link);
 
     if ( e ==  e_NO_EXCEPTION )  
@@ -216,6 +257,9 @@ RTIG::processJoinFederation(Socket *link, NetworkMessage *req)
         strcpy(repFED.FEDid,filename) ;   
 
         // Send answer
+
+        G.Out(pdGendoc,"processJoinFederation====>End  FED file transfer");
+
         repFED.write(link);
         }
     // END of FED file processing
@@ -230,7 +274,14 @@ RTIG::processJoinFederation(Socket *link, NetworkMessage *req)
           federate, num_federation, num_federe);
 
     // Send answer
+
+    G.Out(pdGendoc,"processJoinFederation====>write");
+
     rep.write(link);
+
+    G.Out(pdGendoc,"exit RTIG::processJoinFederation");
+    G.Out(pdGendoc,"END ** JOIN FEDERATION SERVICE **");
+
 }
 
 // ----------------------------------------------------------------------------
@@ -239,9 +290,17 @@ void
 RTIG::processResignFederation(Handle federation,
                               FederateHandle federe)
 {
+
+    G.Out(pdGendoc,"BEGIN ** RESIGN FEDERATION SERVICE **");
+    G.Out(pdGendoc,"enter RTIG::processResignFederation");
+
     federations.remove(federation, federe);
     D.Out(pdInit, "Federate %u is resigning from federation %u.",
           federe, federation);
+
+    G.Out(pdGendoc,"exit RTIG::processResignFederation");
+    G.Out(pdGendoc,"END ** RESIGN FEDERATION SERVICE **");
+
 }
 
 // ----------------------------------------------------------------------------
@@ -252,6 +311,9 @@ RTIG::processDestroyFederation(Socket *link, NetworkMessage *req)
     Handle num_federation ;
     char *federation = req->federationName ;
 
+    G.Out(pdGendoc,"BEGIN ** DESTROY FEDERATION SERVICE **");
+    G.Out(pdGendoc,"enter RTIG::processDestroyFederation");
+
     if (federation == NULL) throw RTIinternalError("Invalid Federation Name.");
 
     auditServer << "Name \"" << federation << "\"" ;
@@ -265,7 +327,13 @@ RTIG::processDestroyFederation(Socket *link, NetworkMessage *req)
     rep.exception = e_NO_EXCEPTION ;
     rep.federate = req->federate ;
 
+    G.Out(pdGendoc,"processDestroyFederation===>write");
+
     rep.write(link);
+
+    G.Out(pdGendoc,"exit RTIG::processDestroyFederation");
+    G.Out(pdGendoc,"END ** DESTROY FEDERATION SERVICE **");
+
 }
 
 // ----------------------------------------------------------------------------
@@ -335,14 +403,28 @@ RTIG::processMessageNull(NetworkMessage *msg)
 void
 RTIG::processRegisterSynchronization(Socket *link, NetworkMessage *req)
 {
+
+    G.Out(pdGendoc,"BEGIN ** REGISTER FEDERATION SYNCHRONIZATION POINT Service **");
+    G.Out(pdGendoc,"enter RTIG::processRegisterSynchronization");
+
     auditServer << "Label \"" << req->label << "\" registered. Tag is \""
 		<< req->tag << "\"" ;
-    
-    federations.manageSynchronization(req->federation,
-                                       req->federate,
-                                       true,
-                                       req->label,
-                                       req->tag);
+
+    // boolean true means a federates set exists
+    if ( req->boolean )
+        federations.manageSynchronization(req->federation,
+                                          req->federate,
+                                          true,
+                                          req->label,
+                                          req->tag,
+                                          req->handleArraySize,
+                                          req->handleArray);
+    else
+        federations.manageSynchronization(req->federation,
+                                          req->federate,
+                                          true,
+                                          req->label,
+                                          req->tag);
     D.Out(pdTerm, "Federation %u is now synchronizing.", req->federation);
 
     // send synchronizationPointRegistrationSucceeded() to federate.
@@ -351,12 +433,28 @@ RTIG::processRegisterSynchronization(Socket *link, NetworkMessage *req)
     rep.federate = req->federate ;
     rep.federation = req->federation ;
     rep.setLabel(req->label);
+
+    G.Out(pdGendoc,"      processRegisterSynchronization====> write SPRS to RTIA");
+
     rep.write(link);
 
-    federations.broadcastSynchronization(req->federation,
+    // boolean true means a federates set exists
+    if ( req->boolean )
+        federations.broadcastSynchronization(req->federation,
+                                          req->federate,
+                                          req->label,
+                                          req->tag,
+                                          req->handleArraySize,
+                                          req->handleArray);
+    else
+        federations.broadcastSynchronization(req->federation,
                                           req->federate,
                                           req->label,
                                           req->tag);
+
+    G.Out(pdGendoc,"exit  RTIG::processRegisterSynchronization");
+    G.Out(pdGendoc,"END   ** REGISTER FEDERATION SYNCHRONIZATION POINT Service **");
+
 }
 
 // ----------------------------------------------------------------------------
@@ -378,10 +476,22 @@ RTIG::processSynchronizationAchieved(Socket *, NetworkMessage *req)
 void
 RTIG::processRequestFederationSave(Socket *, NetworkMessage *req)
 {
+    G.Out(pdGendoc,"BEGIN ** REQUEST FEDERATION SAVE SERVICE **");
+    G.Out(pdGendoc,"enter RTIG::processRequestFederationSave");
+
     auditServer << "Federation save request." ;
 
-    federations.requestFederationSave(req->federation, req->federate,
-                                       req->label, req->date);
+    if ( req->boolean )
+        // With time
+        federations.requestFederationSave(req->federation, req->federate,
+                                          req->label, req->date);
+    else
+        // Without time
+        federations.requestFederationSave(req->federation, req->federate,
+                                          req->label);
+
+    G.Out(pdGendoc,"exit  RTIG::processRequestFederationSave");
+    G.Out(pdGendoc,"END   ** REQUEST FEDERATION SAVE SERVICE **");
 }
 // ----------------------------------------------------------------------------
 void
@@ -574,11 +684,14 @@ RTIG::processUpdateAttributeValues(Socket *link, NetworkMessage *req)
     auditServer << "ObjID = " << req->object
 		<< ", Date = " << req->date ;
 
-    // Prepare le Value Array
+    // Get Value Array
     ValueArray = req->getAttribValueArray();
 
-    // Propage l'appel
-    federations.updateAttribute(req->federation,
+    // Forward the call
+    if ( req->getBoolean() )
+        {
+        // UAV with time
+        federations.updateAttribute(req->federation,
                                  req->federate,
                                  req->object,
                                  req->handleArray,
@@ -586,9 +699,21 @@ RTIG::processUpdateAttributeValues(Socket *link, NetworkMessage *req)
                                  req->handleArraySize,
                                  req->date,
                                  req->label);
+        }
+    else
+        {
+        // UAV without time
+        federations.updateAttribute(req->federation,
+                                 req->federate,
+                                 req->object,
+                                 req->handleArray,
+                                 ValueArray,
+                                 req->handleArraySize,
+                                 req->label);
+        }
     free(ValueArray);
 
-    // Prepare la reponse
+    // Building answer (Network Message re)
     NetworkMessage rep ;
     rep.type = NetworkMessage::UPDATE_ATTRIBUTE_VALUES ;
     rep.exception = e_NO_EXCEPTION ;
@@ -596,6 +721,9 @@ RTIG::processUpdateAttributeValues(Socket *link, NetworkMessage *req)
     rep.object = req->object ;
     rep.date = req->date ;
     rep.handleArraySize = 0 ;
+    // Don't forget label and tag
+    strcpy(rep.label,req->label) ;
+    strcpy(rep.tag,req->tag) ;
 
     rep.write(link); // send answer to RTIA
 }
@@ -607,12 +735,17 @@ RTIG::processSendInteraction(Socket *link, NetworkMessage *req)
 {
     ValueLengthPair *values = NULL ;
 
-    // Prepare le Value Array
+    G.Out(pdGendoc,"BEGIN ** SEND INTERACTION SERVICE **");
+    G.Out(pdGendoc,"enter RTIG::processSendInteraction");
+
+    // Building Value Array
     auditServer << "IntID = " << req->interactionClass
 		<< ", date = " << req->date ;
     values = req->getParamValueArray();
 
-    federations.updateParameter(req->federation,
+    if ( req->getBoolean() )
+        {
+        federations.updateParameter(req->federation,
 				req->federate,
 				req->interactionClass,
 				req->handleArray,
@@ -621,9 +754,21 @@ RTIG::processSendInteraction(Socket *link, NetworkMessage *req)
 				req->date,
 				req->region,
 				req->label);
+        }
+    else
+        {
+        federations.updateParameter(req->federation,
+				req->federate,
+				req->interactionClass,
+				req->handleArray,
+				values,
+				req->handleArraySize,
+				req->region,
+				req->label);
+        }
     free(values);
 
-    D.Out(pdDebug, "Mise A Jour des parametres de l'interaction %d terminee",
+    D.Out(pdDebug, "Interaction %d parameters update completed",
           req->interactionClass);
 
     NetworkMessage rep ;
@@ -632,8 +777,15 @@ RTIG::processSendInteraction(Socket *link, NetworkMessage *req)
     rep.federate = req->federate ;
     rep.interactionClass = req->interactionClass ;
     rep.handleArraySize = 0 ;
-
+    // Don't forget label and tag
+    strcpy(rep.label,req->label) ;
+    strcpy(rep.tag,req->tag) ;
+    G.Out(pdGendoc,"processSendInteraction===>write");
     rep.write(link); // send answer to RTIA
+
+    G.Out(pdGendoc,"exit RTIG::processSendInteraction");
+    G.Out(pdGendoc,"END ** SEND INTERACTION SERVICE **");
+
 }
 
 // ----------------------------------------------------------------------------
@@ -1122,6 +1274,14 @@ void
 RTIG::processRegisterObjectWithRegion(Socket *link, NetworkMessage *req)
 {
     NetworkMessage rep ;
+    // FIXME bug #9869
+    // When we were passed a set of region
+    // we should register object for each region  
+    // the trouble comes from the fact that
+    // there is both
+    //     - req->region  (coming from NetworkMessage::region)
+    //     - req->regions (coming from BasicMessage::regions)
+    // would be nice to sort those thing out.
     rep.object = federations.registerObjectWithRegion(req->federation,
 						      req->federate,
 						      req->objectClass,
@@ -1144,4 +1304,4 @@ RTIG::processRegisterObjectWithRegion(Socket *link, NetworkMessage *req)
 
 }} // namespace certi/rtig
 
-// $Id: RTIG_processing.cc,v 3.33 2007/04/20 08:27:07 rousse Exp $
+// $Id: RTIG_processing.cc,v 3.44 2007/11/16 15:04:22 rousse Exp $
