@@ -18,9 +18,9 @@
 // ----------------------------------------------------------------------------
 // $Id
 
-#include <config.h>
-#include "Message.hh"
 
+
+#include "Message.hh"
 #include "certi.hh"
 #include "PrettyDebug.hh"
 #include "GAV.hh"
@@ -65,10 +65,21 @@ Message::Message()
     number = 0 ;
     region = 0 ;
     FEDid[0] = '\0' ;
-
+    for ( int i=0 ; i<MAX_ATTRIBUTES_PER_CLASS ; i++ )
+        {
+        valueArray[i].length = 0 ;
+        valueArray[i].value[0]  = '\0' ;
+        }
 }
-
-//getValue allows Value as a set of bytes because length (parameter 2) is known
+/** getValue : Value and its length are tooken from valueArray[Rank]
+    @param Rank valueArray rank
+    @param length byte number of Value tooken from valueArray[Rank] (modified)
+    @param Value Value tooken from valueArray[Rank]
+           Value is managed as a set of bytes
+    Note : if Value is not present, Value is created and its address is returned
+           by getValue
+    getValue allows Value as a set of bytes because length (parameter 2) is known
+*/
 // ----------------------------------------------------------------------------
 char *Message::getValue(int Rank, unsigned long *length, char *Value) const
     throw (RTIinternalError)
@@ -98,6 +109,8 @@ char *Message::getValue(int Rank, unsigned long *length, char *Value) const
 }
 
 // ----------------------------------------------------------------------------
+/** Return a newly allocated ValueArray, exactly of size HandleArraySize.
+*/
 ParameterLengthPair *
 Message::getValueArray()
 {
@@ -122,13 +135,22 @@ Message::getValueArray()
 }
 
 // ----------------------------------------------------------------------------
+/** set label into a Message
+    @param NewLabel copied (strcpy) into Message label (NULL label is stored as an empty string)
+*/
 void
 Message::setLabel(const char *NewLabel)
 {
-    if (strlen(NewLabel) > MAX_USER_TAG_LENGTH)
-        throw ValueLengthExceeded("Label too long to fit in Message.");
-
-    strcpy(label, NewLabel);
+    if ( NewLabel )
+        {
+        if (strlen(NewLabel) > MAX_USER_TAG_LENGTH)
+            throw ValueLengthExceeded("Label too long to fit in Message.");
+        strcpy(label, NewLabel);
+        }
+    else
+        {
+        strcpy(label, "") ;
+        }
 }
 
 // ----------------------------------------------------------------------------
@@ -211,6 +233,9 @@ Message::setFederationTimeDelta(FederationTimeDelta the_lookahead)
 }
 
 // ----------------------------------------------------------------------------
+/** Store FedTime into a Message
+    @param the_time federation time
+*/
 void
 Message::setFedTime(const RTI::FedTime &the_time)
 {
@@ -232,6 +257,9 @@ Message::setFederationTime(FederationTime the_time)
 }
 
 // ----------------------------------------------------------------------------
+/** Store a boolean into a Message
+    @param[in] the_bool boolean
+*/
 void
 Message::setBoolean(bool the_bool)
 {
@@ -239,6 +267,9 @@ Message::setBoolean(bool the_bool)
 }
 
 // ----------------------------------------------------------------------------
+/** Store ObjectHandle into a Message
+    @param the_object object handle
+*/
 void
 Message::setObject(ObjectHandle the_object)
 {
@@ -349,6 +380,9 @@ Message::getAHVPS() const
 }
 
 // ----------------------------------------------------------------------------
+/** store attribute set into handleArray and valueArray of a Message
+    @param the_attributes attribute set
+*/
 void
 Message::setAHVPS(const RTI::AttributeHandleValuePairSet &the_attributes)
 {
@@ -480,7 +514,9 @@ Message::setFederateName(const char *NewNomFedere)
 
 // ----------------------------------------------------------------------------
 // setTag
-//
+/** Store Tag into a Message (i.e. in tag, size is limited by MAX_USER_TAG_LENGTH)
+    @param new_tag tag (NULL tag is stored as an empty string)
+*/
 void
 Message::setTag(const char *new_tag)
 {
@@ -601,6 +637,8 @@ Message::display(char *s)
     printf(" -- MESSAGE - %s -", s);
     if ( type == CREATE_FEDERATION_EXECUTION )
       printf("CREATE_FEDERATION_EXECUTION : federation %s : filename %s\n",federationName,FEDid) ;
+    if ( type == DESTROY_FEDERATION_EXECUTION )
+      printf("DESTROY_FEDERATION_EXECUTION : federation %s : \n",federationName) ;
     else if ( type == JOIN_FEDERATION_EXECUTION )
       printf("JOIN_FEDERATION_EXECUTION\n") ;
     else
