@@ -16,12 +16,12 @@
 // License along with this program ; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 //
-// $Id: NetworkMessage_RW.cc,v 3.24 2007/04/27 16:24:50 erk Exp $
+// $Id: NetworkMessage_RW.cc,v 3.32 2007/11/13 13:25:40 rousse Exp $
 // ----------------------------------------------------------------------------
 
-#include <config.h>
-#include "NetworkMessage.hh"
 
+
+#include "NetworkMessage.hh"
 #include "PrettyDebug.hh"
 
 using std::vector ;
@@ -30,7 +30,14 @@ using std::cout;
 
 namespace certi {
 
-static pdCDebug D("RTIG_MSG", __FILE__);
+static pdCDebug D("RTIG_MSG","(NetWorkMS) - ");
+static PrettyDebug G("GENDOC",__FILE__);
+
+// ----------------------------------------------------------------------------
+void NetworkMessage::trace(const char* context)
+{
+D.Mes(pdMessage,'N',this->type,context);
+}
 
 // ----------------------------------------------------------------------------
 // readBody
@@ -45,14 +52,23 @@ NetworkMessage::readBody(Socket *socket)
 
     // 1. Read Body from socket.
     socket->receive((void *) body.getBuffer(), Header.bodySize);
-
+    // FIXME EN: we must update the write pointer of the 
+	 //           MessageBody because we have just written 
+	 //           on it using direct pointer access !! (nasty usage)
+	 body.addToWritePointer(Header.bodySize);
+	 
     // 3. Read informations from Message Body according to message type.
+	//D.Mes(pdMessage, 'N',Header.type);
+	this->trace("NetworkMessage::readBody ");
+
+    if (Header.exception != e_NO_EXCEPTION) {
+        body.readString(exceptionReason, MAX_EXCEPTION_REASON_LENGTH);
+    }
+    else {
     switch(Header.type) {
       case GET_FED_FILE:
         number = body.readShortInt();
-        unsigned short nb;      
-        nb = body.readShortInt();
-        body.readBlock(FEDid,nb);
+        body.readString(FEDid,MAX_FEDFILE_NAME_LENGTH);
         if ( number >= 1 )  // open (0) and close (0) no more information
             {
             ValueArray[0].length = body.readLongInt();
@@ -61,9 +77,20 @@ NetworkMessage::readBody(Socket *socket)
 	break ;
             
       case UPDATE_ATTRIBUTE_VALUES:
+	object = body.readLongInt();
+	readLabel(body);
+	boolean = body.readLongInt();   // true means with time
+	body.readBlock((char *) handleArray, handleArraySize * sizeof(AttributeHandle));
+	for (i = 0 ; i < handleArraySize ; i ++) {
+            ValueArray[i].length = body.readLongInt();
+            body.readBlock(ValueArray[i].value, ValueArray[i].length) ;
+	}
+	break ;
+
       case REFLECT_ATTRIBUTE_VALUES:
 	object = body.readLongInt();
 	readLabel(body);
+	boolean = body.readLongInt();    // true means with time
 	body.readBlock((char *) handleArray, handleArraySize * sizeof(AttributeHandle));
 	for (i = 0 ; i < handleArraySize ; i ++) {
             ValueArray[i].length = body.readLongInt();
@@ -75,6 +102,7 @@ NetworkMessage::readBody(Socket *socket)
       case SEND_INTERACTION:
       case RECEIVE_INTERACTION:
 	readLabel(body);
+	boolean = body.readLongInt();   // true means with time
 	body.readBlock((char *) handleArray,
 		       handleArraySize * sizeof(AttributeHandle));
 	for (i = 0 ; i < handleArraySize ; i ++) {
@@ -94,6 +122,18 @@ NetworkMessage::readBody(Socket *socket)
 	break ;
 	
       case REGISTER_FEDERATION_SYNCHRONIZATION_POINT:
+	readLabel(body);
+	readTag(body);
+	boolean = body.readLongInt();
+        // boolean true means there is an handleArray
+        if ( boolean)
+            {
+            handleArraySize = body.readShortInt();
+	    for (i = 0 ; i < handleArraySize ; i ++)
+	        handleArray[i] = body.readShortInt();
+            }
+	break ;
+
       case ANNOUNCE_SYNCHRONIZATION_POINT:
 	readLabel(body);
 	readTag(body);
@@ -102,13 +142,23 @@ NetworkMessage::readBody(Socket *socket)
       case SYNCHRONIZATION_POINT_ACHIEVED:
       case SYNCHRONIZATION_POINT_REGISTRATION_SUCCEEDED:
       case FEDERATION_SYNCHRONIZED:
-      case REQUEST_FEDERATION_SAVE:
-      case INITIATE_FEDERATE_SAVE:
       case REQUEST_FEDERATION_RESTORE:
       case REQUEST_FEDERATION_RESTORE_SUCCEEDED:
       case INITIATE_FEDERATE_RESTORE:
 	readLabel(body);
 	break ;
+
+      case INITIATE_FEDERATE_SAVE:
+	readLabel(body);
+        // boolean true means with time (in the header)
+	boolean = body.readLongInt();
+	break ;
+
+      case REQUEST_FEDERATION_SAVE:
+	readLabel(body);
+        // boolean true means with time (in the header)
+        boolean = body.readLongInt();
+	break ;
 	
       case REQUEST_FEDERATION_RESTORE_FAILED:
 	readLabel(body);
@@ -238,6 +288,7 @@ NetworkMessage::readBody(Socket *socket)
 	D.Out(pdExcept, "Unknown type %d in ReadBody.", Header.type);
 	throw RTIinternalError("Unknown/Unimplemented type for body.");
     }
+    }
 
 }
 
@@ -253,14 +304,22 @@ NetworkMessage::readHeader(Socket *socket)
     exception = Header.exception ;
     federate = Header.federate ;
     federation = Header.federation ;
+    // If the message carry an exception, the Body will only contain the
+    // exception reason.
+
+    if (exception != e_NO_EXCEPTION)
+        return true ;
 
     // 2- Parse Header according to its type(Variable Part)
     switch (Header.type) {
       case MESSAGE_NULL:
-      case REQUEST_FEDERATION_SAVE:
 	date = Header.VP.time.date ;
 	break ;
 
+      case REQUEST_FEDERATION_SAVE:
+	date = Header.VP.O_I.date ;
+	break ;
+
       case UPDATE_ATTRIBUTE_VALUES:
       case REFLECT_ATTRIBUTE_VALUES:
 	objectClass = Header.VP.O_I.handle ;
@@ -276,6 +335,9 @@ NetworkMessage::readHeader(Socket *socket)
 	break ;
 
       case INITIATE_FEDERATE_SAVE:
+	date = Header.VP.O_I.date ;
+	break ;
+
       case REQUEST_FEDERATION_RESTORE:
       case INITIATE_FEDERATE_RESTORE:
       case REQUEST_FEDERATION_RESTORE_SUCCEEDED:
@@ -396,7 +458,6 @@ NetworkMessage::readHeader(Socket *socket)
     }
 
     // 4- If Header.bodySize is not 0, return RTI_TRUE, else RTI_FALSE
-
     return Header.bodySize ;
 }
 
@@ -416,12 +477,19 @@ NetworkMessage::writeBody(Socket *socket)
     D.Out(pdTrace, "HeaderStruct size is : <%d> out of <%d> bytes MAX in body\n",
 	  sizeof(HeaderStruct),BUFFER_SIZE_DEFAULT);
 
+   // If the message carry an exception, the Body will only contain the
+   // exception reason.
+
+   if (Header.exception != e_NO_EXCEPTION) {
+        body.writeString(exceptionReason);
+   }
+   else
+    {
     // 1- Prepare body Structure according to Message type
     switch(Header.type) {
       case GET_FED_FILE:
         body.writeShortInt(number);
-        body.writeShortInt(strlen(FEDid));
-        body.writeBlock(FEDid,strlen(FEDid));
+        body.writeString(FEDid);
         if ( number >= 1 )  // open (0) and close (0) no more information
             {
             body.writeLongInt(ValueArray[0].length);
@@ -430,9 +498,21 @@ NetworkMessage::writeBody(Socket *socket)
       break;
 
       case UPDATE_ATTRIBUTE_VALUES:
+	body.writeLongInt(object);
+	body.writeString(label);
+	body.writeLongInt(boolean);    // true means with time (stored in header)
+	body.writeBlock((char *) handleArray, handleArraySize * sizeof(AttributeHandle));
+	
+	for (i = 0 ; i < handleArraySize ; i ++) {
+            body.writeLongInt(ValueArray[i].length) ;
+            body.writeBlock(ValueArray[i].value, ValueArray[i].length);
+	}
+	break ;
+
       case REFLECT_ATTRIBUTE_VALUES:
 	body.writeLongInt(object);
 	body.writeString(label);
+        body.writeLongInt(boolean);
 	body.writeBlock((char *) handleArray, handleArraySize * sizeof(AttributeHandle));
 	
 	for (i = 0 ; i < handleArraySize ; i ++) {
@@ -446,6 +526,7 @@ NetworkMessage::writeBody(Socket *socket)
       case SEND_INTERACTION:
       case RECEIVE_INTERACTION:
 	body.writeString(label);
+	body.writeLongInt(boolean);    // true means with time (stored in header)
 	body.writeBlock((char *) handleArray,
 			handleArraySize * sizeof(AttributeHandle));
 	for (i = 0 ; i < handleArraySize ; i ++) {
@@ -467,22 +548,44 @@ NetworkMessage::writeBody(Socket *socket)
 	break ;
 
       case REGISTER_FEDERATION_SYNCHRONIZATION_POINT:
-      case ANNOUNCE_SYNCHRONIZATION_POINT:
 	body.writeString(label);
 	body.writeString(tag);
+	body.writeLongInt(boolean);
+        // boolean true means we have an handleArray
+        if ( boolean )
+            {
+	    body.writeShortInt(handleArraySize);
+	    for (i = 0 ; i < handleArraySize ; i ++)
+	        body.writeShortInt(handleArray[i]);
+            }        
+	break ;
+
+      case ANNOUNCE_SYNCHRONIZATION_POINT:
+	body.writeString(label);
+	body.writeString(tag);       
 	break ;
 
       case SYNCHRONIZATION_POINT_ACHIEVED:
       case SYNCHRONIZATION_POINT_REGISTRATION_SUCCEEDED:
       case FEDERATION_SYNCHRONIZED:
-      case REQUEST_FEDERATION_SAVE:
-      case INITIATE_FEDERATE_SAVE:
       case REQUEST_FEDERATION_RESTORE:
       case REQUEST_FEDERATION_RESTORE_SUCCEEDED:
       case INITIATE_FEDERATE_RESTORE:
 	body.writeString(label);
 	break ;
 
+      case INITIATE_FEDERATE_SAVE:
+	body.writeString(label);
+        // boolean true means with time (in the header)
+        body.writeLongInt(boolean);
+	break ;
+
+      case REQUEST_FEDERATION_SAVE:
+        body.writeString(label);
+        // boolean true means with time (in the header)
+        body.writeLongInt(boolean);
+	break ;
+
       case REQUEST_FEDERATION_RESTORE_FAILED:
 	body.writeString(label);
 	body.writeString(tag);
@@ -611,12 +714,14 @@ NetworkMessage::writeBody(Socket *socket)
 	D.Out(pdExcept, "Unknown type %d in Writebody.", Header.type);
 	throw RTIinternalError("Unknown/Unimplemented type for Header.");
     }
+ }
 
     // body Size does not include the copy of the Header!
     Header.bodySize = body.size() - sizeof(HeaderStruct);
 
     // Put the real body Size in the copy of the Header.
-    (reinterpret_cast<HeaderStruct *>(body.getBufferRW()))->bodySize = Header.bodySize ;
+    // FIXME do we really need the body size in the header??
+    (reinterpret_cast<HeaderStruct *>(body.getBufferModeRW()))->bodySize = Header.bodySize ;
     D.Out(pdTrace,"Sending MessageBody of size <%d>",body.size());
     socket->send(body.getBuffer(), body.size());
 }
@@ -630,6 +735,13 @@ NetworkMessage::writeHeader(Socket *socket)
     Header.exception = exception ;
     Header.federate = federate ;
     Header.federation = federation ;
+    // If the message carry an exception, the Body will only contain the
+    // exception reason.
+
+    if (exception != e_NO_EXCEPTION) {
+        Header.bodySize = 1 ;
+        return true ;
+    }
 
     // 3- Fill Header(Variable Part)[Sorted by Variable part type]
     // Note: Header.bodySize is not set to the actual body size, but
@@ -641,6 +753,12 @@ NetworkMessage::writeHeader(Socket *socket)
 	break ;
 	
       case UPDATE_ATTRIBUTE_VALUES:
+	Header.bodySize = 1 ;
+        Header.VP.O_I.handle = objectClass ;
+        Header.VP.O_I.size = handleArraySize ;
+        Header.VP.O_I.date = date ;
+	break ;
+
       case REFLECT_ATTRIBUTE_VALUES:
 	Header.bodySize = 1 ;
         Header.VP.O_I.handle = objectClass ;
@@ -659,10 +777,18 @@ NetworkMessage::writeHeader(Socket *socket)
 
       case REQUEST_FEDERATION_SAVE:
 	Header.bodySize = 1 ;
-	Header.VP.O_I.date = date ;
+        // boolean true means with time
+        if ( boolean)
+	    Header.VP.O_I.date = date ;
 	break ;
 
       case INITIATE_FEDERATE_SAVE:
+        // boolean true means with time
+        if ( boolean)
+	    Header.VP.O_I.date = date ;
+	Header.bodySize = 1 ;
+	break ;
+
       case REQUEST_FEDERATION_RESTORE:
       case REQUEST_FEDERATION_RESTORE_SUCCEEDED:
       case REQUEST_FEDERATION_RESTORE_FAILED:
@@ -837,4 +963,4 @@ NetworkMessage::writeHeader(Socket *socket)
 
 } // namespace certi
 
-// $Id: NetworkMessage_RW.cc,v 3.24 2007/04/27 16:24:50 erk Exp $
+// $Id: NetworkMessage_RW.cc,v 3.32 2007/11/13 13:25:40 rousse Exp $
