@@ -19,18 +19,21 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA
 //
-// $Id: ObjectClassSet.cc,v 3.22 2007/05/03 15:46:31 rousse Exp $
+// $Id: ObjectClassSet.cc,v 3.29 2007/11/19 09:29:03 erk Exp $
 // ----------------------------------------------------------------------------
 
-#include "ObjectClassSet.hh"
-
 // Project
-#include <config.h>
+#include "Object.hh"
+#include "ObjectClass.hh"
+#include "ObjectClassSet.hh"
 #include "ObjectClassBroadcastList.hh"
+#include "SecurityServer.hh"
 #include "PrettyDebug.hh"
+#include "Named.hh"
 
 // Standard
 #include <iostream>
+#include <sstream>
 
 using std::list ;
 using std::cout ;
@@ -39,18 +42,40 @@ using std::string ;
 
 namespace certi {
 
-static pdCDebug D("OBJECTCLASSSET", __FILE__);
+static PrettyDebug D("OBJECTCLASSSET", __FILE__);
 
 // ----------------------------------------------------------------------------
 //! The class is not allocated, only the pointer is memorized.
 void
-ObjectClassSet::addClass(ObjectClass *newClass)
+ObjectClassSet::addClass(ObjectClass *newClass) throw (RTIinternalError)
 {
+	Name2ObjectClassMap_t::iterator findit;
+	std::stringstream       msg;
+	
     D.Out(pdInit, "Adding new object class %d.", newClass->getHandle());
 
+    /* link to server */
     newClass->server = server ;
-
-    push_front(newClass);
+    /* 
+     * Check whether addition of this object clas
+     * will generate a name collision or not.
+     * i.e. we may not add an object class of the SAME
+     * name to the object class set
+     */
+    findit = OCFromName.find(newClass->getName());
+    if (findit != OCFromName.end()) {
+    	msg << "Name collision another object class named <"
+    	    << newClass->getName()
+    	    << "> with handle <"
+    	    << findit->second->getHandle()
+    	    << "> was found when trying to add identically named object class with handle <"
+    	    << newClass->getHandle();
+    	throw RTIinternalError(msg.str().c_str());
+    }
+    /* store ref to new class in ObjectClass from Handle Map */
+    OCFromHandle[newClass->getHandle()] = newClass;
+    /* store ref to new class in ObjectClass from Name Map */
+    OCFromName[newClass->getName()] = newClass;
 }
 
 // ----------------------------------------------------------------------------
@@ -61,28 +86,48 @@ ObjectClassSet::buildParentRelation(ObjectClass *subclass,
 				    ObjectClass *superclass)
 {
     subclass->setSuperclass(superclass->getHandle());
-    subclass->setLevelId(superclass->getLevelId());
+    subclass->setLevelId(superclass->getLevelId());    
     superclass->addSubclass(subclass);
     superclass->addAttributesToChild(subclass);
 }
 
 // ----------------------------------------------------------------------------
 //! Constructor.
-ObjectClassSet::ObjectClassSet(SecurityServer *theSecurityServer)
-    : list<ObjectClass *>()
+ObjectClassSet::ObjectClassSet(SecurityServer *theSecurityServer, bool isRootClassSet)   
+
 {
     // It can be NULL on the RTIA.
-    server = theSecurityServer ;
+    this->server = theSecurityServer ;
+    this->isRootClassSet = isRootClassSet;
 }
 
 // ----------------------------------------------------------------------------
 //! Destructor.
 ObjectClassSet::~ObjectClassSet()
 {
-    while (!empty()) {
-        delete front();
-        pop_front();
-    }
+	/* clear name map */
+	OCFromName.clear();    
+	/* 
+	 * If we are Root ClassSet (the class set owned by RootObject)
+	 *    we delete the content
+	 * If not we only clear the map in order to avoid double deletion.
+	 * 	
+	 * FIXME EN: this is a trick in order because we do not
+	 *           really maintain a tree of ObjectClass in order
+	 *           to support flat object class name 
+	 *           ("Boule" instead of "Bille.Boule")
+	 *           We may get rid of this as soon as we want to support
+	 *           same name for object class in different branch of the tree.
+	 */
+	if (isRootClassSet) {
+		while (!OCFromHandle.empty()) {
+			delete (OCFromHandle.begin()->second);
+			OCFromHandle.erase(OCFromHandle.begin());
+		}
+	}
+	else {
+		OCFromHandle.clear();
+	}
 }
 
 // ----------------------------------------------------------------------------
@@ -136,9 +181,9 @@ ObjectClassSet::display() const
 {
     cout << " ObjectClasses :" << endl ;
 
-    list<ObjectClass *>::const_iterator i ;
-    for (i = begin(); i != end(); i++) {
-        (*i)->display();
+    handledOC_const_iterator i;    
+    for (i = OCFromHandle.begin(); i != OCFromHandle.end(); ++i) {
+        i->second->display();
     }
 }
 
@@ -189,15 +234,17 @@ ObjectClass *
 ObjectClassSet::getInstanceClass(ObjectHandle theObjectHandle) const
     throw (ObjectNotKnown)
 {
-    list<ObjectClass *>::const_iterator i ;
-    for (i = begin(); i != end(); i++) {
-        if ((*i)->isInstanceInClass(theObjectHandle) == true)
-            return (*i);
+	std::stringstream msg;
+	
+	handledOC_const_iterator i ;
+    for (i = OCFromHandle.begin(); i != OCFromHandle.end(); ++i) {
+        if (i->second->isInstanceInClass(theObjectHandle) == true)
+            return (i->second);
     }
 
-    D.Out(pdExcept, "Object instance %d not found in any object class.",
-          theObjectHandle);
-    throw ObjectNotKnown("");
+	msg << "ObjectHandle <" << theObjectHandle <<"> not found in any object class.";
+    D.Out(pdExcept, msg.str().c_str());
+    throw ObjectNotKnown(msg.str().c_str());
 }
 
 // ----------------------------------------------------------------------------
@@ -207,10 +254,12 @@ Object *
 ObjectClassSet::getObject(ObjectHandle h) const
     throw (ObjectNotKnown)
 {
-    list<ObjectClass *>::const_iterator i ;
-    for (i = begin(); i != end(); i++) {
+	
+	handledOC_const_iterator i ;
+	
+    for (i = OCFromHandle.begin(); i != OCFromHandle.end(); ++i) {
 	try {
-	    Object *object = (*i)->getInstanceWithID(h);
+	    Object *object = i->second->getInstanceWithID(h);
 	    return object ;
 	}
 	catch (ObjectNotKnown &e) {
@@ -224,14 +273,54 @@ ObjectClassSet::getObject(ObjectHandle h) const
 ObjectClassHandle
 ObjectClassSet::getObjectClassHandle(std::string class_name) const
     throw (NameNotFound)
-{
-    list<ObjectClass *>::const_iterator i ;
-    for (i = begin(); i != end(); i++) {
-        if ((*i)->getName() == class_name)
-            return (*i)->getHandle();
+{    
+    std::string currentName;
+    std::string remainingName;
+    ObjectClassHandle currentHandle;
+    ObjectClass*      currentClass;
+    ObjectClassSet const*   currentClassSet;
+	namedOC_const_iterator iter;
+	
+	currentClassSet = this;
+	remainingName = class_name;
+    /* 
+     * If the name is qualified (a.k.a. hierarchical name)
+     * like "Bille.Boule"
+     * then iterate through subClass in order to reach the leaf
+     * "unqualified name"
+     */
+    while (Named::isQualifiedClassName(remainingName)) {
+    	/* 
+    	 * The first current should be the name of
+    	 * of a subclass of the current ObjectClassSet 
+    	 */    	
+    	currentName = Named::getNextClassName(remainingName);
+		/* 
+		 * Get the handle of the subclass 
+		 * NOTE that we won't recurse more than once here
+		 * since the provided 'currentName' is not qualified
+		 * 'by design'
+		 * The recursive deepness is at most 2. 
+		 */
+		currentHandle = currentClassSet->getObjectClassHandle(currentName);
+		/* Get the corresponding class object */
+		currentClass = currentClassSet->getWithHandle(currentHandle);
+		/* now update currentClassSet */
+		currentClassSet = currentClass->getSubClasses();       	 
     }
-    throw NameNotFound("");
-}
+    
+    /* 
+     * Now the current classClassSet should be a leaf 
+     * so that we can search in the 
+     */           
+    iter = currentClassSet->OCFromName.find(remainingName);
+
+	if (iter != currentClassSet->OCFromName.end()) {
+		return iter->second->getHandle();
+	} else {
+		throw NameNotFound(class_name.c_str());
+	}
+}  
 
 // ----------------------------------------------------------------------------
 //! getObjectClassName.
@@ -251,15 +340,17 @@ ObjectClass *
 ObjectClassSet::getWithHandle(ObjectClassHandle theHandle) const
     throw (ObjectClassNotDefined)
 {
-    list<ObjectClass *>::const_iterator i ;
-    for (i = begin(); i != end(); i++) {
-        if ((*i)->getHandle() == theHandle)
-            return (*i);
-    }
-
-    D.Out(pdExcept, "Unknown Object Class Handle %d .", theHandle);
-
-    throw ObjectClassNotDefined("Unknow class handle.");
+	
+	handledOC_const_iterator iter;
+	
+	iter = OCFromHandle.find(theHandle);
+		
+	if (iter != OCFromHandle.end()) {
+		return iter->second;
+	} else {
+		D.Out(pdExcept, "Unknown Object Class Handle %d .", theHandle);
+		throw ObjectClassNotDefined("Unknow class handle.");
+	}		    
 }
 
 // ----------------------------------------------------------------------------
@@ -267,21 +358,22 @@ ObjectClassSet::getWithHandle(ObjectClassHandle theHandle) const
 void ObjectClassSet::killFederate(FederateHandle theFederate)
     throw ()
 {
-    ObjectClassBroadcastList *ocbList = NULL ;
-    ObjectClassHandle currentClass = 0 ;
-
-    list<ObjectClass *>::iterator i ;
-    for (i = begin(); i != end(); i++) {
+    ObjectClassBroadcastList *ocbList      = NULL ;
+    ObjectClassHandle         currentClass = 0 ;
+    
+    Handle2ObjectClassMap_t::iterator i;
+    
+    for (i = OCFromHandle.begin(); i != OCFromHandle.end(); ++i) {
         // Call KillFederate on that class until it returns NULL.
         do {
             D.Out(pdExcept, "Kill Federate Handle %d .", theFederate);
-            ocbList = (*i)->killFederate(theFederate);
+            ocbList = i->second->killFederate(theFederate);
 
             D.Out(pdExcept, "Federate Handle %d Killed.", theFederate);
 
             // Broadcast RemoveObject message recursively
             if (ocbList != 0) {
-                currentClass = (*i)->getSuperclass();
+                currentClass = i->second->getSuperclass();
                 D.Out(pdExcept, "List not NULL");
                 while (currentClass != 0) {
                     D.Out(pdRegister,
@@ -289,11 +381,11 @@ void ObjectClassSet::killFederate(FederateHandle theFederate)
                           currentClass);
 
                     // It may throw ObjectClassNotDefined
-                    (*i) = getWithHandle(currentClass);
+                    i->second = getWithHandle(currentClass);
 
-                    (*i)->broadcastClassMessage(ocbList);
+                    i->second->broadcastClassMessage(ocbList);
 
-                    currentClass = (*i)->getSuperclass();
+                    currentClass = i->second->getSuperclass();
                 }
 
                 delete ocbList ;
@@ -407,7 +499,7 @@ ObjectClassSet::subscribe(FederateHandle federate,
 }
 
 // ----------------------------------------------------------------------------
-//! updateAttributeValues.
+//! updateAttributeValues with time
 void
 ObjectClassSet::updateAttributeValues(FederateHandle federate,
                                       ObjectHandle object_handle,
@@ -452,6 +544,51 @@ ObjectClassSet::updateAttributeValues(FederateHandle federate,
     delete ocbList ;
 }
 
+// ----------------------------------------------------------------------------
+//! updateAttributeValues without time
+void
+ObjectClassSet::updateAttributeValues(FederateHandle federate,
+                                      ObjectHandle object_handle,
+                                      AttributeHandle *attributes,
+                                      ValueLengthPair *values,
+                                      UShort nb,
+                                      const char *tag)
+    throw (ObjectNotKnown,
+           AttributeNotDefined,
+           AttributeNotOwned,
+           RTIinternalError,
+           InvalidObjectHandle)
+{
+    Object *object = getObject(object_handle);
+    ObjectClass *object_class = getWithHandle(object->getClass());
+    ObjectClassHandle current_class = object_class->getHandle();
+
+    D.Out(pdProtocol, "Federate %d Updating object %d from class %d.",
+          federate, object_handle, current_class);
+
+    // It may throw a bunch of exceptions
+    ObjectClassBroadcastList *ocbList = NULL ;
+    ocbList = object_class->updateAttributeValues(
+	federate, object, attributes, values, nb, tag);
+
+    // Broadcast ReflectAttributeValues message recursively
+    current_class = object_class->getSuperclass();
+
+    while (current_class != 0) {
+        D.Out(pdProtocol,
+              "Broadcasting RAV msg to parent class %d for instance %d.",
+              current_class, object_handle);
+
+        // It may throw ObjectClassNotDefined
+        object_class = getWithHandle(current_class);
+        object_class->broadcastClassMessage(ocbList, object);
+
+        current_class = object_class->getSuperclass();
+    }
+
+    delete ocbList ;
+}
+
 // ----------------------------------------------------------------------------
 //! negotiatedAttributeOwnershipDivestiture.
 void
@@ -642,4 +779,4 @@ cancelAttributeOwnershipAcquisition(FederateHandle theFederateHandle,
 
 } // namespace certi
 
-// $Id: ObjectClassSet.cc,v 3.22 2007/05/03 15:46:31 rousse Exp $
+// $Id: ObjectClassSet.cc,v 3.29 2007/11/19 09:29:03 erk Exp $
