@@ -19,7 +19,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA
 //
-// $Id: RTIambPrivateRefs.cc,v 3.0 2005/04/30 17:39:18 breholee Exp $
+// $Id: RTIambPrivateRefs.cc,v 3.6 2007/11/16 15:04:22 rousse Exp $
 // ----------------------------------------------------------------------------
 
 #include <config.h>
@@ -30,6 +30,15 @@ namespace {
 PrettyDebug D("LIBRTI", __FILE__);
 }
 
+RTIambPrivateRefs::RTIambPrivateRefs()
+{
+	fed_amb      = NULL;
+	pid_RTIA     = (pid_t)-1;
+	is_reentrant = false;
+	_theRootObj  = NULL;
+	socketUn     = NULL;
+}
+
 RTIambPrivateRefs::~RTIambPrivateRefs()
 {
     delete socketUn ;
@@ -39,7 +48,7 @@ RTIambPrivateRefs::~RTIambPrivateRefs()
 void
 RTIambPrivateRefs::leave(const char *msg)
 {
-    std::cout << "libRTI: " << msg << std::endl ;
+    std::cout << "LibRTI:: " << msg << std::endl ;
     exit(EXIT_FAILURE);
 }
 
@@ -48,8 +57,12 @@ void
 RTIambPrivateRefs::executeService(Message *req, Message *rep)
 {
     // raise exception if reentrant call.
-    if (is_reentrant)
-        throw ConcurrentAccessAttempted("");
+	// FIXME EN: On SMP machine may we really 
+	//           guarantee that the following protection
+	//           against re-entrance is an efficient one?
+    if (is_reentrant) {
+        throw ConcurrentAccessAttempted("is_reentrant was true in RTIambPrivateRefs::executeService");
+    }
 
     D.Out(pdDebug, "sending request to RTIA.");
 
@@ -95,7 +108,7 @@ RTIambPrivateRefs::executeService(Message *req, Message *rep)
 void
 RTIambPrivateRefs::processException(Message *msg)
 {
-    D.Out(pdExcept, "n de l'exception : %d .", msg->getExceptionType());
+    D.Out(pdExcept, "num de l'exception : %d .", msg->getExceptionType());
     switch(msg->getExceptionType()) {
       case e_NO_EXCEPTION: {
       } break ;
@@ -105,6 +118,16 @@ RTIambPrivateRefs::processException(Message *msg)
           throw ArrayIndexOutOfBounds(msg->getExceptionReason());
       } break ;
 
+      case e_AsynchronousDeliveryAlreadyEnabled: {
+          D.Out(pdExcept, "Throwing e_AsynchronousDeliveryAlreadyEnabled exception.");
+          throw AsynchronousDeliveryAlreadyEnabled(msg->getExceptionReason());
+      } break ;
+
+      case e_AsynchronousDeliveryAlreadyDisabled: {
+          D.Out(pdExcept, "Throwing e_AsynchronousDeliveryAlreadyDisabled exception.");
+          throw AsynchronousDeliveryAlreadyDisabled(msg->getExceptionReason());
+      } break ;
+
       case e_AttributeAlreadyOwned: {
           D.Out(pdExcept, "Throwing e_AttributeAlreadyOwned exception.");
           throw AttributeAlreadyOwned(msg->getExceptionReason());
@@ -169,6 +192,11 @@ RTIambPrivateRefs::processException(Message *msg)
           throw CouldNotDiscover(msg->getExceptionReason());
       } break ;
 
+      case e_CouldNotOpenFED: {
+          D.Out(pdExcept, "Throwing e_CouldNotOpenFED exception.");
+          throw CouldNotOpenFED(msg->getExceptionReason());
+      } break ;
+
       case e_CouldNotOpenRID: {
           D.Out(pdExcept, "Throwing e_CouldNotOpenRID exception.");
           throw CouldNotOpenRID(msg->getExceptionReason());
@@ -189,6 +217,11 @@ RTIambPrivateRefs::processException(Message *msg)
           throw ErrorReadingRID(msg->getExceptionReason());
       } break ;
 
+      case e_ErrorReadingFED: {
+          D.Out(pdExcept, "Throwing e_ErrorReadingFED exception.");
+          throw ErrorReadingFED(msg->getExceptionReason());
+      } break ;
+
       case e_EventNotKnown: {
           D.Out(pdExcept, "Throwing e_EventNotKnown exception.");
           throw EventNotKnown(msg->getExceptionReason());
@@ -509,4 +542,4 @@ RTIambPrivateRefs::processException(Message *msg)
     }
 }
 
-// $Id: RTIambPrivateRefs.cc,v 3.0 2005/04/30 17:39:18 breholee Exp $
+// $Id: RTIambPrivateRefs.cc,v 3.6 2007/11/16 15:04:22 rousse Exp $
