@@ -19,10 +19,10 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA
 //
-// $Id: RTIambassador.cc,v 3.40 2007/05/03 15:46:31 rousse Exp $
+// $Id: RTIambassador.cc,v 3.57 2007/11/13 13:25:40 rousse Exp $
 // ----------------------------------------------------------------------------
 
-#include <config.h>
+
 #include "certi.hh"
 
 #include "RTIambPrivateRefs.hh"
@@ -31,9 +31,15 @@
 #include "Message.hh"
 #include "PrettyDebug.hh"
 
-#include <signal.h>
-#include <iostream>
+#ifdef _WIN32
+#include <config.h>
+#include <stdio.h>
+#include <string.h>
+#else 
 #include <unistd.h>
+#endif
+#include <iostream>
+#include <signal.h>
 #include <cassert>
 
 using std::cout ;
@@ -43,6 +49,7 @@ using std::endl ;
 namespace {
 
 static pdCDebug D("LIBRTI", __FILE__);
+static PrettyDebug G("GENDOC",__FILE__) ;
 
 using namespace certi ;
 
@@ -52,13 +59,13 @@ build_region_handles(RTI::Region **regions, int nb)
 {
     std::vector<RTI::Handle> vect(nb);
     for (int i = 0 ; i < nb ; ++i) {
-	RTI::Region *region = regions[i] ;
-	try {
-	    vect[i] = dynamic_cast<FedRegion *>(region)->getHandle();
-	}
-	catch (std::bad_cast) {
-	    throw RTI::RegionNotKnown("");
-	}
+    	RTI::Region *region = regions[i] ;
+    	try {
+    		vect[i] = dynamic_cast<FedRegion *>(region)->getHandle();
+    	}
+    	catch (std::bad_cast) {
+    		throw RTI::RegionNotKnown("");
+    	}
     }
     return vect ;
 }
@@ -84,6 +91,7 @@ get_handle(const RTI::Region &region)
   is launched. This process is used for data exchange with rtig server.
   This process connects to rtia after one second delay (UNIX socket).
 */
+
 RTI::RTIambassador::RTIambassador()
     throw (MemoryExhausted, RTIinternalError)
 {
@@ -99,8 +107,40 @@ RTI::RTIambassador::RTIambassador()
     const char *rtiacall ;
     if (rtiaenv) rtiacall = rtiaenv ;
     else rtiacall = rtiaexec ;
+    
+#ifdef _WIN32
+  STARTUPINFO si;
+  PROCESS_INFORMATION pi;
 
-    // creating RTIA process.
+  ZeroMemory( &si, sizeof(si) );
+  si.cb = sizeof(si);
+  ZeroMemory( &pi, sizeof(pi) );
+
+  // Start the child process. 
+  if( !CreateProcess( NULL, // No module name (use command line). 
+        (char*)rtiacall,	// Command line. 
+        NULL,					// Process handle not inheritable. 
+        NULL,					// Thread handle not inheritable. 
+        FALSE,					// Set handle inheritance to FALSE. 
+        0,   					// No creation flags. 
+        NULL,					// Use parent's environment block. 
+        NULL,					// Use parent's starting directory. 
+        &si,					// Pointer to STARTUPINFO structure.
+        &pi ))					// Pointer to PROCESS_INFORMATION structure.
+				 
+		{
+		printf("Error code : %d\n", GetLastError());
+		errno = GetLastError();
+		perror("CreateProcess");
+		exit(-1);
+		}
+    
+   privateRefs->handle_RTIA = pi.hProcess;
+   privateRefs->pid_RTIA = pi.dwProcessId;
+
+  sleep(1);
+  privateRefs->socketUn->connectUN(privateRefs->pid_RTIA);
+#else
     switch((privateRefs->pid_RTIA = fork())) {
       case -1: // fork failed.
         perror("fork");
@@ -127,6 +167,7 @@ RTI::RTIambassador::RTIambassador()
         };
         break ;
     }
+#endif
 }
 
 // ----------------------------------------------------------------------------
@@ -136,8 +177,12 @@ RTI::RTIambassador::RTIambassador()
 RTI::RTIambassador::~RTIambassador()
     throw (RTIinternalError)
 {
-    kill(privateRefs->pid_RTIA, SIGINT);
-    delete privateRefs ;
+#ifdef _WIN32
+	TerminateProcess(privateRefs->handle_RTIA, 1);
+#else
+	kill(privateRefs->pid_RTIA, SIGINT);
+#endif
+delete privateRefs ;
 }
 
 // ----------------------------------------------------------------------------
@@ -147,6 +192,30 @@ RTI::RTIambassador::tick()
     throw (SpecifiedSaveLabelDoesNotExist,
            ConcurrentAccessAttempted,
            RTIinternalError)
+{
+   // non-blocking tick, returns immediately with or without 1 callback
+   return tick_kernel (false);
+}
+
+// ----------------------------------------------------------------------------
+// Tick
+RTI::Boolean
+RTI::RTIambassador::tick2()
+    throw (SpecifiedSaveLabelDoesNotExist,
+           ConcurrentAccessAttempted,
+           RTIinternalError)
+{
+   // blocking tick, waits until a callback delivery, be careful
+   return tick_kernel (true);
+}
+
+// ----------------------------------------------------------------------------
+// Tick_kernel
+RTI::Boolean
+RTI::RTIambassador::tick_kernel(bool locked)
+    throw (SpecifiedSaveLabelDoesNotExist,
+           ConcurrentAccessAttempted,
+           RTIinternalError)
 {
     Message vers_RTI, vers_Fed ;
 
@@ -158,15 +227,19 @@ RTI::RTIambassador::tick()
 
     // Prevenir le RTI
     vers_RTI.type = Message::TICK_REQUEST ;
+    if (locked)
+       vers_RTI.setBoolean(true);
+    else
+       vers_RTI.setBoolean(false);
 
     try {
         vers_RTI.write(privateRefs->socketUn);
     }
     catch (NetworkError) {
         cout << "tick 1." << endl ;
-        cout << "LibRTI: Catched NetworkError, throw RTIinternalError."
+        cout << "LibRTI:: Catched NetworkError, throw RTIinternalError."
              << endl ;
-        throw RTIinternalError("");
+        throw RTIinternalError("RTI::RTIambassador::tick_kernel (tick1) : NetworkError-->RTIinternalError");
     }
 
     for (;;) {
@@ -177,9 +250,9 @@ RTI::RTIambassador::tick()
         }
         catch (NetworkError) {
             cout << "tick 2." << endl ;
-            cout << "LibRTI: Catched NetworkError, throw RTIinternalError."
+            cout << "LibRTI:: Catched NetworkError, throw RTIinternalError."
                  << endl ;
-            throw RTIinternalError("");
+            throw RTIinternalError("RTI::RTIambassador::tick_kernel (tick2) : NetworkError-->RTIinternalError");
         }
 
         // Si c'est de type TICK_REQUEST, il n'y a qu'a traiter l'exception.
@@ -193,17 +266,19 @@ RTI::RTIambassador::tick()
         // du FederateAmbassador.
 
         vers_RTI.setException(e_NO_EXCEPTION);
+
         try {
+				//D.Mes( pdMessage, 'M', vers_Fed.type);
+				vers_Fed.trace("RTI::RTIambassador::tick ");
+				
             switch (vers_Fed.type) {
 
               case Message::SYNCHRONIZATION_POINT_REGISTRATION_SUCCEEDED:
-                privateRefs->fed_amb->synchronizationPointRegistrationSucceeded(
-		    vers_Fed.getLabel());
+                privateRefs->fed_amb->synchronizationPointRegistrationSucceeded(vers_Fed.getLabel());
                 break ;
 
               case Message::ANNOUNCE_SYNCHRONIZATION_POINT:
-                privateRefs->fed_amb->announceSynchronizationPoint(vers_Fed.getLabel(),
-                                                      vers_Fed.getTag());
+                privateRefs->fed_amb->announceSynchronizationPoint(vers_Fed.getLabel(),vers_Fed.getTag());
                 break ;
 
               case Message::FEDERATION_SYNCHRONIZED:
@@ -272,27 +347,39 @@ RTI::RTIambassador::tick()
 
               case Message::REFLECT_ATTRIBUTE_VALUES: {
                   AttributeHandleValuePairSet *attributes = vers_Fed.getAHVPS();
-                  privateRefs->fed_amb->
-                      reflectAttributeValues(vers_Fed.getObject(),
-                                             *attributes,
-                                             vers_Fed.getFedTime(),
-                                             vers_Fed.getTag(),
-                                             vers_Fed.getEventRetraction());
-
+                  if (vers_Fed.getBoolean())
+                     privateRefs->fed_amb->
+                         reflectAttributeValues(vers_Fed.getObject(),
+                                                *attributes,
+                                                vers_Fed.getFedTime(),
+                                                vers_Fed.getTag(),
+                                                vers_Fed.getEventRetraction());
+                  else
+                     privateRefs->fed_amb->
+                         reflectAttributeValues(vers_Fed.getObject(),
+                                                *attributes,
+                                                vers_Fed.getTag());
                   delete attributes ;
               } break ;
 
-              case Message::RECEIVE_INTERACTION: {
+              case Message::RECEIVE_INTERACTION:
+                  {
                   ParameterHandleValuePairSet *parameters = vers_Fed.getPHVPS();
-
-                  privateRefs->fed_amb->receiveInteraction(vers_Fed.getInteractionClass(),
+                  if (vers_Fed.getBoolean())
+                      privateRefs->fed_amb->receiveInteraction(
+                                              vers_Fed.getInteractionClass(),
                                               *parameters,
                                               vers_Fed.getFedTime(),
                                               vers_Fed.getTag(),
                                               vers_Fed.getEventRetraction());
+                  else
+                      privateRefs->fed_amb->receiveInteraction(
+                                              vers_Fed.getInteractionClass(),
+                                              *parameters,
+                                              vers_Fed.getTag());
 
                   delete parameters ;
-              } break ;
+                  } break ;
 
               case Message::REMOVE_OBJECT_INSTANCE: {
                   privateRefs->fed_amb->removeObjectInstance(vers_Fed.getObject(),
@@ -421,9 +508,9 @@ RTI::RTIambassador::tick()
         }
         catch (NetworkError) {
             cout << "tick 3." << endl ;
-            cout << "LibRTI: Catched NetworkError, throw RTIinternalError."
+            cout << "LibRTI:: Catched NetworkError, throw RTIinternalError."
                  << endl ;
-            throw RTIinternalError("");
+            throw RTIinternalError("RTI::RTIambassador::tick_kernel (tick 3) : NetworkError-->RTIinternalError");
         }
     }
 }
@@ -439,28 +526,52 @@ RTI::RTIambassador::tick(TickTime, TickTime)
 
 
 // ----------------------------------------------------------------------------
-//! Create Federation Execution.
-/*! Send a CREATE_FEDERATION_EXECUTION request type to inform rtia process a
-  new federation is being created.
+// Create Federation Execution.
+/** Realization of the Create Federation Execution federation management service
+    (HLA 1.3).
+    Send a CREATE_FEDERATION_EXECUTION request type to inform rtia process a
+    new federation is being created.
+    @param executionName execution name of the federation to be created
+    @param FED           FED file name (path seen by rtig)
 */
 void
-RTI::RTI::RTIambassador::createFederationExecution(const char *executionName,
-                                         const char *FED)
+//RTI::
+RTI::RTIambassador::createFederationExecution(const char *executionName,
+					      const char *FED)
     throw (RTI::RTIinternalError, RTI::ConcurrentAccessAttempted, 
 	   RTI::ErrorReadingFED, RTI::CouldNotOpenFED, 
 	   RTI::FederationExecutionAlreadyExists)
 {
-    Message req, rep ;
+Message req, rep ;
 
-    req.type = Message::CREATE_FEDERATION_EXECUTION ;
-    req.setFederationName(executionName);
-    req.setFEDid(FED);
+G.Out(pdGendoc,"enter RTIambassador::createFederationExecution");
+
+req.type = Message::CREATE_FEDERATION_EXECUTION ;
+req.setFederationName(executionName);
+req.setFEDid(FED);
+
+/*#ifdef _WIN32
+	if(!stricmp(FED,executionName)) {
+#else
+	if(!strcasecmp(FED,exeName)) {
+#endif
+}*/
+G.Out(pdGendoc,"             ====>executeService CREATE_FEDERATION_EXECUTION");
+
+privateRefs->executeService(&req, &rep);
+
+G.Out(pdGendoc,"exit RTIambassador::createFederationExecution");
 
-    privateRefs->executeService(&req, &rep);
 }
 
 // ----------------------------------------------------------------------------
-//! Destroy Federation Execution.
+// Destroy Federation Execution.
+/** Realization of the Destroy Federation Execution federation management service
+    (HLA 1.3).
+    Send a DESTROY_FEDERATION_EXECUTION request type to remove a federation
+    execution from the RTI set of federation executions.
+    \param executionName execution name of the federation to be destroyed
+*/
 void
 RTI::RTIambassador::destroyFederationExecution(const char *executionName)
     throw (RTI::RTIinternalError, RTI::ConcurrentAccessAttempted,
@@ -468,10 +579,17 @@ RTI::RTIambassador::destroyFederationExecution(const char *executionName)
 {
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::destroyFederationExecution");
+
     req.type = Message::DESTROY_FEDERATION_EXECUTION ;
     req.setFederationName(executionName);
 
+    G.Out(pdGendoc,"        ====>executeService DESTROY_FEDERATION_EXECUTION");
+
     privateRefs->executeService(&req, &rep);
+
+    G.Out(pdGendoc,"exit RTIambassador::destroyFederationExecution");
+
 }
 
 // ----------------------------------------------------------------------------
@@ -486,6 +604,8 @@ RTI::RTIambassador::joinFederationExecution(const char *yourName,
 {
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::joinFederationExecution");
+
     privateRefs->fed_amb = (FederateAmbassador *) fedamb ;
 
     req.type = Message::JOIN_FEDERATION_EXECUTION ;
@@ -493,6 +613,7 @@ RTI::RTIambassador::joinFederationExecution(const char *yourName,
     req.setFederationName(executionName);
 
     privateRefs->executeService(&req, &rep);
+    G.Out(pdGendoc,"exit  RTIambassador::joinFederationExecution");
     return rep.getFederate();
 }
 
@@ -508,13 +629,23 @@ RTI::RTIambassador::resignFederationExecution(ResignAction theAction)
 {
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::resignFederationExecution");
+
     req.type = Message::RESIGN_FEDERATION_EXECUTION ;
     req.setResignAction(theAction);
     privateRefs->executeService(&req, &rep);
+
+    G.Out(pdGendoc,"exit RTIambassador::resignFederationExecution");
+
 }
 
 // ----------------------------------------------------------------------------
-//! Register Federation Synchronization Point
+//! Register Federation Synchronization Point without set of federates 
+//  (all federates in the federation execution are informed)
+/**
+    @param label Synchronization point label
+    @param the_tag User-supplied tag
+*/
 void
 RTI::RTIambassador::registerFederationSynchronizationPoint(const char *label,
                                                       const char *the_tag)
@@ -526,29 +657,64 @@ RTI::RTIambassador::registerFederationSynchronizationPoint(const char *label,
 {
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::registerFederationSynchronizationPoint for all federates");
+
     req.type = Message::REGISTER_FEDERATION_SYNCHRONIZATION_POINT ;
     req.setLabel(label);
     req.setTag(the_tag);
+    // boolean false because without set of federates
+    req.setBoolean(false);
 
     privateRefs->executeService(&req, &rep);
+
+    G.Out(pdGendoc,"exit RTIambassador::registerFederationSynchronizationPoint for all federates");
+
 }
 
 // ----------------------------------------------------------------------------
+//! Register Federation Synchronization Point with set of federates (UNIMPLEMENTED)
+//  (only federates in the set are informed)
+/**
+    @param label Synchronization point label
+    @param theTag User-supplied tag
+    @param set_of_fed set of federate designators
+*/
 void
 RTI::RTIambassador::registerFederationSynchronizationPoint(const char *label,
 						      const char *theTag,
-						      const FederateHandleSet &)
+						      const FederateHandleSet &set_of_fed)
     throw (RTI::RTIinternalError, RTI::RestoreInProgress, RTI::SaveInProgress,
 	   RTI::ConcurrentAccessAttempted, RTI::FederateNotExecutionMember)
 {
-    throw UnimplementedService("");
+
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::registerFederationSynchronizationPoint for some federates");
+
     req.type = Message::REGISTER_FEDERATION_SYNCHRONIZATION_POINT ;
     req.setLabel(label);
     req.setTag(theTag);
+    // boolean must be true because federate set exists but if size=0 (set empty)
+    // we have to put boolean false (HLA 1.3 compliance to inform ALL federates)
+    if ( set_of_fed.size() == 0 )
+        {
+        req.setBoolean(false) ;
+        }
+    else
+        {
+        req.setBoolean(true);
+        // and then we have to store this set (after the size)
+        req.handleArraySize = set_of_fed.size() ;
+        for (unsigned long i=0 ; i<set_of_fed.size() ; i++)
+            {
+            req.handleArray[i] = set_of_fed.getHandle(i) ;
+            }
+        }
 
     privateRefs->executeService(&req, &rep);
+
+    G.Out(pdGendoc,"exit RTIambassador::registerFederationSynchronizationPoint for some federates");
+
 }
 
 // ----------------------------------------------------------------------------
@@ -564,14 +730,18 @@ RTI::RTIambassador::synchronizationPointAchieved(const char *label)
 {
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::synchronizationPointAchieved");
+
     req.type = Message::SYNCHRONIZATION_POINT_ACHIEVED ;
     req.setLabel(label);
 
     privateRefs->executeService(&req, &rep);
+
+    G.Out(pdGendoc,"exit  RTIambassador::synchronizationPointAchieved");
 }
 
 // ----------------------------------------------------------------------------
-//! Request Federation Save.
+//! Request Federation Save with time.
 void
 RTI::RTIambassador::requestFederationSave(const char *label,
                                      const FedTime& theTime)
@@ -583,19 +753,26 @@ RTI::RTIambassador::requestFederationSave(const char *label,
            RestoreInProgress,
            RTIinternalError)
 {
-    throw UnimplementedService("");
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::requestFederationSave with time");
+
     req.type = Message::REQUEST_FEDERATION_SAVE ;
     req.setFedTime(theTime);
     req.setLabel(label);
+    // boolean true means time constrained
     req.setBoolean(true);
 
+    G.Out(pdGendoc,"      requestFederationSave===>executeService "
+                   "REQUEST_FEDERATION_SAVE");
+
     privateRefs->executeService(&req, &rep);
+
+    G.Out(pdGendoc,"exit RTIambassador::requestFederationSave with time");
 }
 
 // ----------------------------------------------------------------------------
-//! Request Federation Save.
+//! Request Federation Save without time.
 void
 RTI::RTIambassador::requestFederationSave(const char *label)
     throw (FederateNotExecutionMember,
@@ -604,14 +781,20 @@ RTI::RTIambassador::requestFederationSave(const char *label)
            RestoreInProgress,
            RTIinternalError)
 {
-    throw UnimplementedService("");
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::requestFederationSave without time");
+
     req.type = Message::REQUEST_FEDERATION_SAVE ;
     req.setLabel(label);
+    // boolean false means without time
     req.setBoolean(false);
 
+    G.Out(pdGendoc,"      ====>executeService for REQUEST_FEDERATION_SAVE");
+
     privateRefs->executeService(&req, &rep);
+
+    G.Out(pdGendoc,"exit  RTIambassador::requestFederationSave without time");
 }
 
 // ----------------------------------------------------------------------------
@@ -624,12 +807,15 @@ RTI::RTIambassador::federateSaveBegun()
            RestoreInProgress,
            RTIinternalError)
 {
-    throw UnimplementedService("");
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::federateSaveBegun");
+
     req.type = Message::FEDERATE_SAVE_BEGUN ;
 
     privateRefs->executeService(&req, &rep);
+
+    G.Out(pdGendoc,"exit  RTIambassador::federateSaveBegun");
 }
 
 // ----------------------------------------------------------------------------
@@ -645,9 +831,12 @@ RTI::RTIambassador::federateSaveComplete()
     throw UnimplementedService("");
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::federateSaveComplete");
+
     req.type = Message::FEDERATE_SAVE_COMPLETE ;
 
     privateRefs->executeService(&req, &rep);
+    G.Out(pdGendoc,"exit  RTIambassador::federateSaveComplete");
 }
 
 // ----------------------------------------------------------------------------
@@ -663,9 +852,13 @@ RTI::RTIambassador::federateSaveNotComplete()
     throw UnimplementedService("");
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::federateSaveNotComplete");
+
     req.type = Message::FEDERATE_SAVE_NOT_COMPLETE ;
 
     privateRefs->executeService(&req, &rep);
+
+    G.Out(pdGendoc,"exit  RTIambassador::federateSaveNotComplete");
 }
 
 // ----------------------------------------------------------------------------
@@ -678,10 +871,13 @@ RTI::RTIambassador::requestFederationRestore(const char *label)
     throw UnimplementedService("");
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::requestFederationRestore");
+
     req.type = Message::REQUEST_FEDERATION_RESTORE ;
     req.setLabel(label);
 
     privateRefs->executeService(&req, &rep);
+    G.Out(pdGendoc,"exit  RTIambassador::requestFederationRestore");
 }
 
 // ----------------------------------------------------------------------------
@@ -695,9 +891,12 @@ RTI::RTIambassador::federateRestoreComplete()
     throw UnimplementedService("");
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::federateRestoreComplete");
+
     req.type = Message::FEDERATE_RESTORE_COMPLETE ;
 
     privateRefs->executeService(&req, &rep);
+    G.Out(pdGendoc,"exit  RTIambassador::federateRestoreComplete");
 }
 
 // ----------------------------------------------------------------------------
@@ -711,9 +910,12 @@ RTI::RTIambassador::federateRestoreNotComplete()
     throw UnimplementedService("");
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::federateRestoreNotComplete");
+
     req.type = Message::FEDERATE_RESTORE_NOT_COMPLETE ;
 
     privateRefs->executeService(&req, &rep);
+    G.Out(pdGendoc,"exit  RTIambassador::federateRestoreNotComplete");
 }
 
 // ----------------------------------------------------------------------------
@@ -728,11 +930,14 @@ RTI::RTIambassador::publishObjectClass(ObjectClassHandle theClass,
 {
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::publishObjectClass");
+
     req.type = Message::PUBLISH_OBJECT_CLASS ;
     req.setObjectClass(theClass);
     req.setAHS(attributeList);
 
     privateRefs->executeService(&req, &rep);
+    G.Out(pdGendoc,"exit  RTIambassador::publishObjectClass");
 }
 
 // ----------------------------------------------------------------------------
@@ -746,9 +951,12 @@ RTI::RTIambassador::unpublishObjectClass(ObjectClassHandle theClass)
 {
     Message req, rep ;
 
+    G.Out(pdGendoc,"enter RTIambassador::unpublishObjectClass");
+
     req.type = Message::UNPUBLISH_OBJECT_CLASS ;
     req.setObjectClass(theClass);
     privateRefs->executeService(&req, &rep);
+    G.Out(pdGendoc,"exit  RTIambassador::unpublishObjectClass");
 }
 
 
@@ -931,8 +1139,6 @@ RTI::RTIambassador::updateAttributeValues(ObjectHandle the_object,
 	   RTI::ConcurrentAccessAttempted, RTI::FederateNotExecutionMember, 
 	   RTI::AttributeNotOwned, RTI::AttributeNotDefined, RTI::ObjectNotKnown)
 {
-    throw UnimplementedService("");
-
     Message req, rep ;
 
     req.type = Message::UPDATE_ATTRIBUTE_VALUES ;
@@ -945,7 +1151,14 @@ RTI::RTIambassador::updateAttributeValues(ObjectHandle the_object,
 }
 
 // ----------------------------------------------------------------------------
-// Send Interaction
+/** Send Interaction with time
+    This service (HLA 1.3) send an interaction into the federation.
+    As the federation time argument is supplied, an event retraction designator is returned.
+    @param theInteraction Interaction class designator
+    @param theParameters Set of interaction parameters designator and value pairs
+    @param theTime Federation time
+    @param theTag User-supplied tag
+*/
 EventRetractionHandle
 RTI::RTIambassador::sendInteraction(InteractionClassHandle theInteraction,
                                const ParameterHandleValuePairSet& theParameters,
@@ -977,23 +1190,29 @@ RTI::RTIambassador::sendInteraction(InteractionClassHandle theInteraction,
 }
 
 // ----------------------------------------------------------------------------
+/** Send Interaction without time
+    This service (HLA 1.3) send an interaction into the federation.
+    None is returned.
+    @param theInteraction Interaction class designator
+    @param theParameters Set of interaction parameters designator and value pairs
+    @param theTag User-supplied tag
+*/
 void
-RTI::RTIambassador::sendInteraction(InteractionClassHandle the_interaction,
-                               const ParameterHandleValuePairSet &parameters,
-                               const char *the_tag)
+RTI::RTIambassador::sendInteraction(InteractionClassHandle theInteraction,
+                               const ParameterHandleValuePairSet &theParameters,
+                               const char *theTag)
     throw (RTI::RTIinternalError, RTI::RestoreInProgress, RTI::SaveInProgress, 
 	   RTI::ConcurrentAccessAttempted, RTI::FederateNotExecutionMember, 
 	   RTI::InteractionParameterNotDefined, RTI::InteractionClassNotPublished, 
 	   RTI::InteractionClassNotDefined)
 {
-    throw UnimplementedService("");
 
     Message req, rep ;
 
     req.type = Message::SEND_INTERACTION ;
-    req.setInteractionClass(the_interaction);
-    req.setTag(the_tag);
-    req.setPHVPS(parameters);
+    req.setInteractionClass(theInteraction);
+    req.setTag(theTag);
+    req.setPHVPS(theParameters);
     req.setRegion(0);
     req.setBoolean(false);
 
@@ -1558,7 +1777,7 @@ RTI::RTIambassador::enableAsynchronousDelivery()
 	   RTI::ConcurrentAccessAttempted, RTI::FederateNotExecutionMember, 
 	   RTI::AsynchronousDeliveryAlreadyEnabled)
 {
-    throw AsynchronousDeliveryAlreadyEnabled("Default value (non HLA)");
+    // throw AsynchronousDeliveryAlreadyEnabled("Default value (non HLA)");
 
     Message req, rep ;
 
@@ -1575,7 +1794,6 @@ RTI::RTIambassador::disableAsynchronousDelivery()
 	   RTI::ConcurrentAccessAttempted, RTI::FederateNotExecutionMember, 
 	   RTI::AsynchronousDeliveryAlreadyDisabled)
 {
-    throw RTIinternalError("Unimplemented Service");
     Message req, rep ;
 
     req.type = Message::DISABLE_ASYNCHRONOUS_DELIVERY ;
@@ -2654,4 +2872,4 @@ RTI::RTIambassador::disableInteractionRelevanceAdvisorySwitch()
     privateRefs->executeService(&req, &rep);
 }
 
-// $Id: RTIambassador.cc,v 3.40 2007/05/03 15:46:31 rousse Exp $
+// $Id: RTIambassador.cc,v 3.57 2007/11/13 13:25:40 rousse Exp $
