@@ -19,12 +19,14 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 // ----------------------------------------------------------------------------
 
-#include "Billard.hh"
-
 #include <config.h>
+
+#include "Billard.hh"
 #include "PrettyDebug.hh"
 
+#ifndef _MSC_VER
 #include <unistd.h>
+#endif
 
 using std::auto_ptr ;
 using std::string ;
@@ -46,6 +48,7 @@ Billard::Billard(std::string federate_name)
       nbTicks(0),
       regulating(false),
       constrained(false),
+      notimestamp(false),      // i.e. with time
       localTime(0.0),
       TIME_STEP(1.0),
       XMAX(500),
@@ -88,10 +91,13 @@ Billard::join(std::string federation_name, std::string fdd_name)
         creator = true ;
     }
     catch (RTI::FederationExecutionAlreadyExists& e) {
+        printf("BILLARD Note : %s Reason is : %s. OK I can join it\n",e._name,e._reason);
         D.Out(pdInit, "Federation execution already created.");
     }
     catch (RTI::CouldNotOpenFED& e) {
+        printf("BILLARD ERROR : %s Reason is : %s\n",e._name,e._reason);
         D.Out(pdExcept, "BILLARD : Could not use FED file.");
+        delete &rtiamb ; // Says RTIA to stop.
         exit(0);
     }
 
@@ -137,8 +143,7 @@ Billard::pause()
     if (creator) {
         D.Out(pdInit, "Pause requested");
         try {
-            rtiamb.registerFederationSynchronizationPoint(
-                "Init", "Waiting all players.");
+            rtiamb.registerFederationSynchronizationPoint("Init", "Waiting all players.");
         }
         catch (RTI::Exception& e) {
             D[pdExcept] << "Federate " << federateName.c_str()
@@ -148,6 +153,37 @@ Billard::pause()
     }
 }
 
+// ----------------------------------------------------------------------------
+/** Creator put federation in pause for synchronization with a friend
+ */
+void
+Billard::pause_friend()
+{
+    if (creator) {
+        D.Out(pdInit, "Pause requested for friend");
+        try {
+            // For testing purpose
+             RTI::FederateHandle numfed ;
+             RTI::FederateHandleSet *federateSet = RTI::FederateHandleSetFactory::create(1) ;
+             cout << "Now we test Register Federation Synchronisation Point on some federates" << endl ;
+             cout << "Please enter a federate handle (zero means none)" << endl ;
+             cout << "This federate will be synchronized with the creator and not the others" << endl;
+             scanf("%d",&numfed) ;
+             if ( numfed != 0 )
+                 {
+                 // We store numfed into the federate set
+                 federateSet->add(numfed) ;
+                 rtiamb.registerFederationSynchronizationPoint("Friend","Synchro with a friend",
+                                                          *federateSet) ;
+                 }
+        }
+        catch (RTI::Exception& e) {
+            D[pdExcept] << "Federate " << federateName.c_str()
+                        << " : Register Synchronization Point failed : %d"
+                        << endl ;
+        }
+    }
+}
 // ----------------------------------------------------------------------------
 /** tick the RTI
  */
@@ -158,9 +194,17 @@ Billard::tick()
     rtiamb.tick();
     nbTicks++ ;
 }
+void
+Billard::tick2()
+{
+    rtiamb.tick2();
+    nbTicks++ ;
+}
 
 // ----------------------------------------------------------------------------
 /** Set time regulation (time regulating and time constrained)
+    @param start_constrained boolean, if true federate is constrained
+    @param start_regulating boolean, if true federate is regulating
  */
 void
 Billard::setTimeRegulation(bool start_constrained, bool start_regulating)
@@ -169,6 +213,7 @@ Billard::setTimeRegulation(bool start_constrained, bool start_regulating)
 
     if (start_constrained) {
         if (!constrained) {
+            // change from no constrained to constrained
             rtiamb.enableTimeConstrained();
             constrained = true ;
             D.Out(pdInit, "Time Constrained enabled.");
@@ -177,6 +222,7 @@ Billard::setTimeRegulation(bool start_constrained, bool start_regulating)
     }
     else {
         if (constrained) {
+            // change from constrained to no constrained
             rtiamb.disableTimeConstrained();
             constrained = false ;
             D.Out(pdInit, "Time Constrained disabled.");
@@ -185,6 +231,7 @@ Billard::setTimeRegulation(bool start_constrained, bool start_regulating)
 
     if (start_regulating) {
         if (!regulating) {
+            // change from no regulating to regulating
             for (;;) {
                 rtiamb.queryFederateTime(localTime);
 
@@ -221,6 +268,7 @@ Billard::setTimeRegulation(bool start_constrained, bool start_regulating)
     }
     else {
         if (regulating) {
+            // change from regulating to no regulating
             rtiamb.disableTimeRegulation();
             regulating = false ;
         }
@@ -271,7 +319,7 @@ Billard::synchronize(int autostart)
             D.Out(pdExcept, "**** Exception achieving a synchronization "
                   "point by creator : %d", &e);
         }
-
+       
         while (paused)
             try {
                 tick();
@@ -390,7 +438,7 @@ Billard::step()
 
     while (!granted) {
         try {
-            tick();
+            tick2();
         }
         catch (RTI::Exception& e) {
             D.Out(pdExcept, "******** Exception ticking the RTI : %d.", &e);
@@ -605,10 +653,17 @@ Billard::sendInteraction(double dx, double dy, const RTI::FedTime& InteractionTi
     D.Out(pdDebug, "SendInteraction - ParamDYID= %u, x= %f, buf= %s",
           ParamDYID, dy, buf);
 
-    D.Out(pdRegister, "Sending interaction(DX= %f, DY= %f).", dx, dy);
-
     try {
-        rtiamb.sendInteraction(BingClassID, *parameterSet, InteractionTime, "");
+        if ( notimestamp )
+            {
+            D.Out(pdRegister, "Sending interaction without time (DX= %f, DY= %f).", dx, dy);
+            rtiamb.sendInteraction(BingClassID, *parameterSet, "");
+            }
+        else
+            {
+            D.Out(pdRegister, "Sending interaction wit time (DX= %f, DY= %f).", dx, dy);
+            rtiamb.sendInteraction(BingClassID, *parameterSet, InteractionTime, "");
+            }
     }
     catch (RTI::Exception& e) {
         D.Out(pdExcept, "**** Exception sending interaction : %d", &e);
@@ -659,10 +714,14 @@ Billard::sendUpdate(double x, double y, int color, const RTI::FedTime& UpdateTim
           AttrColorID, color, attributeSet->size());
 
     try {
-        rtiamb.updateAttributeValues(id, *attributeSet, UpdateTime, "");
-        // if (log)
-        // logfile << string(((RTIfedTime) UpdateTime).getTime()) << " : UAV "
-        // << string(Local.x) << " " << string(Local.y) << endl ;
+        if ( notimestamp )
+            {
+            rtiamb.updateAttributeValues(id, *attributeSet, "coucou");
+            }
+        else
+            {
+            rtiamb.updateAttributeValues(id, *attributeSet, UpdateTime, "coucou");
+            }
     }
     catch (RTI::Exception& e) {
         D.Out(pdExcept, "**** Exception updating attribute values: %d", &e);
@@ -725,6 +784,13 @@ Billard::announceSynchronizationPoint(const char *label, const char */*tag*/)
         paused = true ;
         D.Out(pdProtocol, "announceSynchronizationPoint.");
     }
+#ifdef TEST_RFSP
+    else if (strcmp(label, "Friend") == 0) {
+        std::cout<<"**** I am happy : I have a friend ****"<<std::endl;
+        paused = true ;
+        D.Out(pdProtocol, "announceSynchronizationPoint (friend).");
+    } 
+#endif       
     else {
         cout << "Unexpected synchronization label" << endl ;
         exit(1);
@@ -788,7 +854,7 @@ Billard::receiveInteraction(RTI::InteractionClassHandle theInteraction,
                 // OLD : dx1 = atoi(parmValue);
                 double d_dx1 ;
                 memcpy(&d_dx1,parmValue,valueLength) ;
-                dx1 = d_dx1 ;
+                dx1 = static_cast<int>(d_dx1);
                 // Local.dx = atof(parmValue);
                 D.Out(pdDebug, "receiveInteraction(*) - dx= %s", parmValue);
                 delete[] parmValue ;
@@ -802,7 +868,7 @@ Billard::receiveInteraction(RTI::InteractionClassHandle theInteraction,
                     // OLD : dy1 = atoi(parmValue);
                     double d_dy1 ;
                     memcpy(&d_dy1,parmValue,valueLength) ;
-                    dy1 = d_dy1 ;
+                    dy1 = static_cast<int>(d_dy1);
                     // Local.dy = atof(parmValue);
                     D.Out(pdDebug, "receiveInteraction(*) - dy= %s", parmValue);
                     delete[] parmValue ;
@@ -832,7 +898,7 @@ Billard::receiveInteraction(RTI::InteractionClassHandle theInteraction,
 }
 
 // ----------------------------------------------------------------------------
-/** Callback : reflect attribute values
+/** Callback : reflect attribute values with time
  */
 void
 Billard::reflectAttributeValues(
@@ -846,7 +912,7 @@ Billard::reflectAttributeValues(
            RTI::InvalidFederationTime,
            RTI::FederateInternalError)
 {
-    D.Out(pdTrace, "reflectAttributeValues");
+    D.Out(pdTrace, "reflectAttributeValues with time");
 
     float x1 = 0 ;
     float y1 = 0 ;
@@ -911,6 +977,82 @@ Billard::reflectAttributeValues(
     }
 }
 
+// ----------------------------------------------------------------------------
+/** Callback : reflect attribute values without time
+ */
+void
+Billard::reflectAttributeValues(
+    RTI::ObjectHandle theObject,
+    const RTI::AttributeHandleValuePairSet& theAttributes,
+    const char */*theTag*/)
+    throw (RTI::ObjectNotKnown,
+           RTI::AttributeNotKnown,
+           RTI::FederateInternalError)
+{
+    D.Out(pdTrace, "reflectAttributeValues without time");
+
+    float x1 = 0 ;
+    float y1 = 0 ;
+
+    RTI::ULong valueLength ;
+    char *attrValue ;
+
+    D.Out(pdDebug, "reflectAttributeValues - nb attributs= %d",
+          theAttributes.size());
+
+    for (unsigned int j=0 ; j<theAttributes.size(); j++) {
+
+        RTI::AttributeHandle handle = theAttributes.getHandle(j);
+        valueLength = theAttributes.getValueLength(j);
+        attrValue = new char[valueLength] ;
+        theAttributes.getValue(j, attrValue, valueLength);
+
+        if (handle == AttrXID) {
+            if (attrValue != NULL) {
+                // OLD : x1 = atof(attrValue);
+                double d_x1 ;
+                memcpy(&d_x1,attrValue,valueLength) ;
+                x1 = d_x1 ;
+                delete[] attrValue ;
+            }
+            else
+                D.Out(pdError, "Fed: ERREUR: missing Attribute.");
+        }
+        else if (handle == AttrYID) {
+            if (attrValue != NULL) {
+                // OLD : y1 = atof(attrValue);
+                double d_y1 ;
+                memcpy(&d_y1,attrValue,valueLength) ;
+                y1 = d_y1 ;
+                delete[] attrValue ;
+            }
+            else
+                D.Out(pdError, "Fed: ERREUR: missing Attribute.");
+        }
+        else
+            D.Out(pdError, "Fed: ERREUR: handle inconnu.");
+    }
+    
+    vector<Ball>::iterator it = remote.begin() ;
+    while (it != remote.end() && it->ID != theObject)
+	++it ;
+
+    if (it == remote.end())
+        D.Out(pdError, "Fed: error, id not found (%d).", theObject);
+    else {
+        it->erase();
+
+        float oldx = it->x ;
+        it->x = x1 ;
+        it->dx = it->x - oldx ;
+
+        float oldy = it->y ;
+        it->y = y1 ;
+        it->dy = it->y - oldy ;
+
+	it->active = true ;
+    }
+}
 // ----------------------------------------------------------------------------
 /** Callback : remove object instance
  */
